/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * trunk
 * Copyright (C) Eoin McMahon 2010 <eoinmcmahon0@AT@gmail.com>
 * 
 * trunk is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * trunk is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "m-model.hpp"

bool MModel::row_record::nearmatch(const MModel::row_record & other)
{
	return ((other.timestamp_ == this->timestamp_ ) &&
		(other.uuid_ == this->uuid_ ));
}

bool MModel::row_record::operator==( MModel::row_record & other)
{
	return ((other.timestamp_ == this->timestamp_ ) &&
		(other.uuid_ == this->uuid_ ));
}

MModel::MModel() :
Data(),
host("localhost"),
IPaddress("127.0.0.1"),
row_add_list(),
row_delete_list()
{
}

MModel::MModel(modelmap & inrows) :
Data(),
host("localhost"),
IPaddress("127.0.0.1"),
row_add_list(),
row_delete_list()
{
	modelmap::iterator it;
	for (it = inrows.begin () ; it != inrows.end () ; it++)
	{
		ModelRow & x = it->second;
		ModelRow * pMrow = new ModelRow::ModelRow(IPaddress);
		for ( int count = 0 ; count != Data::n_m_s_cols ; count++ )
		{
			pMrow->set_str_value(Data::m_s_col(count),
								 x.get_str_value(Data::m_s_col(count)));
		}
		for ( int count = 0 ; count != Data::n_m_b_cols ; count++ )
		{
			pMrow->set_boo_value(Data::m_b_col(count),
								 x.get_boo_value(Data::m_b_col(count)));
		}
		std::pair<Data::STR, ModelRow > input(pMrow->get_str_value(Data::uuid), *pMrow);
		rows.insert (input);
	}
}

size_t MModel::size ()
{
	return rows.size ();
}

bool MModel::empty ()
{
	return rows.empty ();
}

MModel::modelmap::iterator MModel::begin ()
{
	return this->rows.begin();
}



MModel::modelmap::iterator MModel::end ()
{
	return this->rows.end ();
}

MModel::modelmap::iterator MModel::find (Data::STR uuid)
{
	// initialise return value
	
	return rows.find (uuid);
}

ModelRow & MModel::operator[](Data::STR uuid)
{
	return rows[uuid];
}

void MModel::insert ( ModelRow & x)
{
	ModelRow * pMrow = new ModelRow::ModelRow(IPaddress);
	for ( int count = 0 ; count != Data::n_m_s_cols ; count++ )
	{
		pMrow->set_str_value(Data::m_s_col(count),
							 x.get_str_value(Data::m_s_col(count)));
	}
	for ( int count = 0 ; count != Data::n_m_b_cols ; count++ )
	{
		pMrow->set_boo_value(Data::m_b_col(count),
							 x.get_boo_value(Data::m_b_col(count)));
	}
	const Data::STR rowuuid = pMrow->get_str_value(Data::uuid);
	std::pair<MModel::modelmap::iterator, bool> ret;
	std::pair<Data::STR, ModelRow > input(pMrow->get_str_value(Data::uuid), *pMrow);
	ret = rows.insert (input);
	
	if (ret.second)
	{
		MModel::emit_row_added(this->get_timestamp(),
							   rowuuid, *pMrow);
	}
	else
	{
		std::cout << "Tried to add duplicate row: " << rowuuid << std::endl;
		for (int i = 0 ; i != Data::n_m_s_cols ; i++ )
		{
			rows[rowuuid].set_str_value(m_s_col(i), 
											  pMrow->get_str_value(m_s_col(i)));
		}
		for (int i = 0 ; i != Data::n_m_b_cols ; i++ )
		{
			rows[rowuuid].set_boo_value(m_b_col(i), 
											  pMrow->get_boo_value(m_b_col(i)));
		}
	}
}
	



void MModel::erase( MModel::modelmap::iterator position)
{
	if (position != rows.end())
	{
		Data::STR rowuuid = position->first;
		ModelRow * pRow = &(position->second);
		rows.erase (position);
		//delete pRow;
		emit_row_deleted(this->get_timestamp(), rowuuid, *pRow);
	}
}

void MModel::clear()
{
	while (!this->empty ())
	{
		erase (rows.begin ());
	}
}

clock_t MModel::get_timestamp()
{
	return clock();
}

MModel::signal_type MModel::row_added_signal()
{
	return m_row_added_signal;
}

MModel::signal_type MModel::row_deleted_signal()
{
	return m_row_deleted_signal;
}
	
void MModel::emit_row_added(clock_t  timestamp, Data::STR  uuid,ModelRow & mrow)
{
	MModel::row_record add_record;
	add_record.timestamp_ = timestamp;
	add_record.uuid_ = uuid;
	add_record.timeout_ = 15;
	row_add_list.push_back(add_record);
	//emit signal
	m_row_added_signal.emit(timestamp, uuid, mrow);
}

void MModel::on_row_added_ack(clock_t  timestamp, Data::STR  uuid, ModelRow & mrow)
{
	//~ MModel::row_record remove_record ;
	//~ remove_record.timestamp_ = timestamp;
	//~ remove_record.uuid_ = uuid;
	//~ remove_record.timeout_ = 15;
	list<MModel::row_record>::iterator it;
	for (it = row_add_list.begin() ; it != row_add_list.end() ; it++)
	{
		if ((it->timestamp_ == timestamp ) && (it->uuid_ == uuid))
		{
			row_add_list.erase(it);
			break;
		}
	}
	//cout<<"RowAddAck received"<<endl;
	//row_add_list.remove(remove_record);
}

void MModel::emit_row_deleted(clock_t  timestamp, Data::STR  uuid, ModelRow & mrow)
{
	MModel::row_record remove_record ;
	remove_record.timestamp_ = timestamp;
	remove_record.uuid_ = uuid;
	remove_record.timeout_ = 15;
	row_delete_list.push_back(remove_record);
	//emit signal
	m_row_deleted_signal.emit(timestamp, uuid, mrow);
}

void MModel::on_row_deleted_ack(clock_t  timestamp, Data::STR  uuid, ModelRow & mrow)
{
	//~ MModel::row_record remove ;
	//~ remove.timestamp_ = timestamp;
	//~ remove.uuid_ = uuid;
	//~ remove.timeout_ = 15;
	//~ row_delete_list.remove_if(remove);
	list<MModel::row_record>::iterator it;
	for (it = row_delete_list.begin() ; it != row_delete_list.end() ; it++)
	{
		if ((it->timestamp_ == timestamp ) && (it->uuid_ == uuid))
		{
			row_delete_list.erase(it);
			break;
		}
	}
	//cout << "RowDeleteAck received" << endl;
	// delete row from memory
	rows.erase(uuid);
}
