/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * trunk
 * Copyright (C) Eoin McMahon 2010 <eoinmcmahon0@AT@gmail.com>
 * 
 * trunk is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * trunk is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _M_MODEL_HPP_
#define _M_MODEL_HPP_


#include <glib.h>
#include <iostream>
#include <vector>
#include <map>
#include <list>
#include <time.h>
#include <gtkmm.h>
#include <libglademm/xml.h>
#include "data.hpp"
#include "model-row.hpp"
#include "connection.hpp"

using namespace std;

		

class MModel : public Data
	{
		public:
		friend class View;
			
			typedef std::pair<Data::STR, ModelRow > 	rowpair;
			typedef map<Data::STR, ModelRow> 			modelmap;
			typedef Connection::sig_type_row 			signal_type;
			
			class row_record
		{
			public:
				clock_t 						timestamp_;
				STR  							uuid_;
				int 							timeout_;
				
				bool 					nearmatch (const row_record & other);
				bool 					operator== (row_record & other);
		};
			
			STR 								host;
			STR 								IPaddress;
			list<row_record> 					row_add_list;
			list<row_record> 					row_delete_list;
			
			MModel();
			MModel(modelmap & inrows);
			
			signal_type 				row_added_signal ();
			signal_type 				row_deleted_signal ();
			modelmap::iterator			begin ();
			modelmap::iterator			end ();
			size_t 						size ();
			bool 						empty ();
			ModelRow & 					operator[] (Data::STR);
			void						insert ( ModelRow & x);
			
			modelmap::iterator			find (Data::STR uuid);
			void			 			erase ( modelmap::iterator position);
			void 						clear ();
			clock_t  					get_timestamp ();
			virtual void 				emit_row_added (clock_t  timestamp, 
											Data::STR  uuid, ModelRow & mrow);
			virtual void 				emit_row_deleted (clock_t  timestamp,
										  Data::STR  uuid, ModelRow & mrow);
			
		
			
		
		
		protected:
			
			signal_type 						m_row_deleted_signal;
			signal_type 						m_row_added_signal;
			modelmap						 	rows;
			
			void 						on_row_added_ack (clock_t  timestamp,
										  Data::STR  uuid, ModelRow & mrow);
			void 						on_row_deleted_ack (clock_t  timestamp,
										  Data::STR  uuid, ModelRow & mrow);
	};

#endif // _M_MODEL_HPP_
