/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * trunk
 * Copyright (C)  2010 <>
	 * 
 * trunk is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
	 * 
 * trunk is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "model-row.hpp"

ModelRow::ModelRow() :
	Data(),
	index(-1),
	myuuid(""),
	m_s_cols(n_m_s_cols),
	m_b_cols(n_m_b_cols),
	changed_str_list(),
	changed_boo_list(),
	parent_host_address(Glib::ustring("no IPaddress"))
{
	
}

ModelRow::ModelRow(Data::STR & IPaddress) :
	Data(),
	index(-1),
	myuuid(""),
	m_s_cols(n_m_s_cols),
	m_b_cols(n_m_b_cols),
	changed_str_list(),
	changed_boo_list(),
	parent_host_address(IPaddress)
{
	
}

clock_t ModelRow::get_timestamp()
{
	return clock();
}

Data::STR ModelRow::get_str_value(m_s_col col)
{
	return m_s_cols[col];
}

Data::BOO ModelRow::get_boo_value(m_b_col col)
{
	return m_b_cols[col];
}

void ModelRow::set_str_value(m_s_col col, Data::STR value)
{
	Data::STR oldvalue = get_str_value(col);
	if (oldvalue != value)
	{
		m_s_cols[col] = value;
		if (col == Data::uuid)
		{
			myuuid = value;
		}
		emit_row_changed_STR (get_timestamp(), get_str_value (uuid), col,
							  oldvalue, value);
	}
	
}

void ModelRow::set_boo_value(m_b_col col, Data::BOO value)
{
	Data::BOO oldvalue = get_boo_value(col);
	if (oldvalue != value)
	{
		m_b_cols[col] = value;
		emit_row_changed_BOO (get_timestamp(), get_str_value (uuid), col,
							  oldvalue, value);
	}
}

vector<Data::STR>::iterator ModelRow::get_str_iterator()
{
	vector<Data::STR>::iterator it;
	it = m_s_cols.begin();
	return it;
}

vector<Data::BOO>::iterator ModelRow::get_boo_iterator()
{
	vector<Data::BOO>::iterator it;
	it = m_b_cols.begin();
	return it;
}

void ModelRow::on_row_changed_STR_ack(clock_t  timestamp,
									  Data::STR uuid,
									  m_s_col col,
									  Data::STR old,
									  Data::STR newvalue)
{
	
	list<ModelRow::changed_str_record>::iterator it;
	for (it = changed_str_list.begin() ; it != changed_str_list.end() ; it++)
	{
		if ((it->timestamp_ == timestamp ) && (it->uuid_ == uuid))
		{
			changed_str_list.erase(it);
			break;
		}
	}
}

ModelRow::type_STR_changed_signal ModelRow::STR_changed_signal()
{
	return m_STR_changed_signal;
}

ModelRow::type_BOO_changed_signal ModelRow::BOO_changed_signal()
{
	return m_BOO_changed_signal;
}

void ModelRow::on_row_changed_BOO_ack(clock_t timestamp,
									  Data::STR uuid,
									  m_b_col col,
									  Data::BOO old,
									  Data::BOO newvalue)
{
	
	list<ModelRow::changed_boo_record>::iterator it;
	for (it = changed_boo_list.begin() ; it != changed_boo_list.end() ; it++)
	{
		if ((it->timestamp_ == timestamp ) && (it->uuid_ == uuid))
		{
			changed_boo_list.erase(it);
			break;
		}
	}
}

void ModelRow::emit_row_changed_STR(clock_t  timestamp,
									Data::STR uuid,
									m_s_col col,
									Data::STR old,
									Data::STR newvalue)
{
	changed_str_record csr;
	csr.timestamp_ = timestamp;
	csr.uuid_ = uuid;
	csr.oldvalue_ = old;
	csr.newvalue_ = newvalue; 
	changed_str_list.push_back(csr);
	//emit signal
	m_STR_changed_signal.emit(timestamp, uuid, col, old, newvalue);
}
void ModelRow::emit_row_changed_BOO(clock_t  timestamp,
									Data::STR uuid,
									m_b_col col,
									Data::BOO old,
									Data::BOO newvalue)
{
	changed_boo_record cbr;
	cbr.timestamp_ = timestamp;
	cbr.uuid_ = uuid;
	cbr.oldvalue_ = old;
	cbr.newvalue_ = newvalue; 
	changed_boo_list.push_back(cbr);
	//emit signal
	m_BOO_changed_signal.emit(timestamp, uuid, col, old, newvalue);
}
