/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * trunk
 * Copyright (C) Eoin McMahon 2010 <eoinmcmahon0@AT@gmail.com>
	 * 
 * trunk is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
	 * 
 * trunk is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <config.h>
#include "vb-manage.hpp"


vbManage::vbManage() : 
	localModel(),
	pView_event_box(),
	pView(),
	timer_count(0),
	about_dialog_loaded(false),
	help_dialog_loaded(false),
	font_dialog_loaded(false),
	colour_dialog_loaded(false)/*,
		 pProgress_bar(),
		 pScr_window(),
		 pStatus_bar(),
		 pWindow(),
		 pQuit_button(),
		 pView(),
		 states(),
		 pMenu_bar(),
		 pFile(),
		 pHelp(),
		 pFile_quit(),
		 pHelp_help(),
		 pHelp_about(),
		 pHelp_help_dialog(),
		 pHelp_about_dialog(),
		 pButton_all_off(),
		 pButton_all_acpi(),
		 pButton_all_save(),
		 pButton_all_pause(),
		 pButton_all_resume(),
		 pButton_all_start(),
		 pButton_all_console()*/
{
	try
	{
		
		
		//		std::cout << "Instantiating a vbManage" << std::endl;
		// set up states map:
		states["running"] = 0;
		states["saved"] = 1;
		states["poweroff"] = 2;
		states["aborted"] = 3;
		states["paused"] = 4;
		states["unknown"] = 5;
		// read in window config from XML:
		try
		{
			refXml = Gnome::Glade::Xml::create(GLADE_FILE_WINDOW);
		}
		catch(const Gnome::Glade::XmlError& ex)
		{
			std::cerr << ex.what() << std::endl;
		}
		//assign pointers to widgets we want to address:
		refXml->get_widget("scrolledwindow1" , pScr_window);
		if (!pScr_window) printf("No ScrollWindow handle!");
		
		
		refXml->get_widget_derived("main_window", pWindow);
		if (!pWindow) printf("No Window handle!\n");
		
		// set window title
		pWindow->set_title(Glib::ustring(PACKAGE_STRING));
		
	 
		//gint width, height, xpos,ypos;
		
		//load values from gconf:
		old_window_width = gconfclient->
			get_int("/apps/vbmanage/mainwindow/width");
		old_window_height = gconfclient->
			get_int("/apps/vbmanage/mainwindow/height");
		old_window_xpos = gconfclient->
			get_int("/apps/vbmanage/mainwindow/xpos");
		old_window_ypos = gconfclient->
			get_int("/apps/vbmanage/mainwindow/ypos");
		bg_window = gconfclient->
			get_string("/apps/vbmanage/mainwindow/colours/bg_window");
		bg_title = gconfclient->
			get_string("/apps/vbmanage/mainwindow/colours/bg_title");
		bg_row_odd = gconfclient->
			get_string("/apps/vbmanage/mainwindow/colours/bg_row_odd");
		bg_row_even = gconfclient->
			get_string("/apps/vbmanage/mainwindow/colours/bg_row_even");
		fontname = gconfclient->
			get_string("/apps/vbmanage/mainwindow/font");
		autorun = gconfclient->
			get_bool("/apps/vbmanage/auto_run");
		show_error_messages = gconfclient->
			get_bool("/apps/vbmanage/show_error_messages");
		
		//get widget for show_error_messages menu item and connect it to handler
		refXml->get_widget("file_show_error_messages", pFile_show_error_messages);
		if (pFile_show_error_messages)
		{
			pFile_show_error_messages->set_active(show_error_messages);
			pFile_show_error_messages->signal_toggled()
				.connect( sigc::mem_fun( *this,
										&vbManage::on_show_error_messages_toggled));
			connection_a = gconfclient->
				notify_add( "/apps/vbmanage/show_error_messages",
						   sigc::mem_fun
						   (*this, &vbManage::on_show_error_messages_external_toggled));
		}
		else
		{
			std::cout << "No File_show_error_messages handle!" << std::endl;
		}
		
		//get widget for autorun menu item and connect it to handler
		refXml->get_widget("file_autorun", pFile_autorun);
		if (pFile_autorun)
		{
			pFile_autorun->set_active(autorun);
			pFile_autorun->signal_toggled()
				.connect( sigc::mem_fun( *this,
										&vbManage::on_autorun_toggled));
			connection_a = gconfclient->
				notify_add( "/apps/vbmanage/auto_run",
						   sigc::mem_fun
						   (*this, &vbManage::on_autorun_external_toggled));
		}
		else
		{
			std::cout << "No File_autorun handle!" << std::endl;
		}
		
		make_autorun_file();
		
		try
		{
			
			// validate window position
			if ( (old_window_xpos < 0 ) || (old_window_xpos > gscreen->get_width()) )
			{
				old_window_xpos = (gconfclient->
								   get_default_from_schema("/apps/vbmanage/mainwindow/xpos")).get_int();
				gconfclient->set("/apps/vbmanage/mainwindow/xpos", (int)old_window_xpos);
			}
			if ( (old_window_ypos < 0 ) || (old_window_ypos > gscreen->get_height()) )
			{
				old_window_ypos = (gconfclient->
								   get_default_from_schema("/apps/vbmanage/mainwindow/ypos")).get_int();
				gconfclient->set("/apps/vbmanage/mainwindow/ypos", (int)old_window_ypos);
			}
			if ( (old_window_width + old_window_xpos > gscreen->get_width()) ||
				(old_window_width < 1) )
			{
				old_window_xpos = (gconfclient->
								   get_default_from_schema("/apps/vbmanage/mainwindow/xpos")).get_int();
				gconfclient->set("/apps/vbmanage/mainwindow/xpos", (int)old_window_xpos);
				old_window_width = (gconfclient->
									get_default_from_schema("/apps/vbmanage/mainwindow/width")).get_int();
				gconfclient->set("/apps/vbmanage/mainwindow/width", (int)old_window_width);
			}
			if ( (old_window_height + old_window_ypos > gscreen->get_height()) ||
				(old_window_height < 1) )
			{
				old_window_ypos = (gconfclient->
								   get_default_from_schema("/apps/vbmanage/mainwindow/ypos")).get_int();
				gconfclient->set("/apps/vbmanage/mainwindow/ypos", (int)old_window_ypos);
				old_window_height = (gconfclient->
									 get_default_from_schema("/apps/vbmanage/mainwindow/height")).get_int();
				gconfclient->set("/apps/vbmanage/mainwindow/height", (int)old_window_height);
			}
			
			// switch for buttons to move to menu
			bool buttons_in_menu = true;
			// instantiate colour buttons and validate colours
			if (buttons_in_menu)
			{
				refXml->get_widget("windowColour", pEdit_window_colour);
				pWindow_colour_button = Gtk::manage(new Gtk::ColorButton());
				pWindow_colour_button->set_title
					(Glib::ustring("Choose a colour for the window background"));
				pWindow_colour_box = Gtk::manage(new Gtk::HBox(false, 10));
				pWindow_colour_label = 
					Gtk::manage(new Gtk::Label
								(Glib::ustring("Window Background ")
								 , Gtk::ALIGN_LEFT));
				pWindow_colour_box->pack_start(*pWindow_colour_button, Gtk::PACK_SHRINK);
				pWindow_colour_box->pack_start(*pWindow_colour_label, Gtk::PACK_SHRINK);
				pEdit_window_colour->remove();
				pEdit_window_colour->add(*pWindow_colour_box);
				pEdit_window_colour->signal_activate().connect
					(sigc::mem_fun(pWindow_colour_button, &Gtk::ColorButton::clicked));
			}
			else
			{
				refXml->get_widget("window_colour_button", pWindow_colour_button);
			}
			if (pWindow_colour_button)
			{
				pWindow_colour_button->set_color(Gdk::Color(bg_window));
				pWindow_colour_button->signal_color_set()
					.connect( sigc::mem_fun( *this,
											&vbManage::on_colour_w_changed));
			}
			else
			{
				std::cout << "No Window colour chooser handle!" << std::endl;
			}
			
			Gdk::Color colour;
			if (!colour.set(bg_window))
			{
				bg_window = (gconfclient->
							 get_default_from_schema
							 ("/apps/vbmanage/mainwindow/colours/bg_window"))
					.get_string();
				gconfclient->set
					("/apps/vbmanage/mainwindow/colours/bg_window", bg_window);
				pWindow_colour_button->set_color(Gdk::Color(bg_window));
			}
			
			if (buttons_in_menu)
			{
				
				refXml->get_widget("titleColour", pEdit_title_colour);
				pTitle_colour_button = Gtk::manage(new Gtk::ColorButton());
				pTitle_colour_button->set_title
					(Glib::ustring("Choose a colour for the title row background"));
				pTitle_colour_box = Gtk::manage(new Gtk::HBox(false, 10));
				pTitle_colour_label = 
					Gtk::manage(new Gtk::Label
								(Glib::ustring("Title row Background ")
								 , Gtk::ALIGN_LEFT));
				pTitle_colour_box->pack_start(*pTitle_colour_button, Gtk::PACK_SHRINK);
				pTitle_colour_box->pack_start(*pTitle_colour_label, Gtk::PACK_SHRINK);
				pEdit_title_colour->remove();
				pEdit_title_colour->add(*pTitle_colour_box);
				pEdit_title_colour->signal_activate().connect
					(sigc::mem_fun(pTitle_colour_button, &Gtk::ColorButton::clicked));
			}
			else
			{
				refXml->get_widget("title_colour_button", pTitle_colour_button);
			}
			if (pTitle_colour_button)
			{
				pTitle_colour_button->set_color(Gdk::Color(bg_title));
				pTitle_colour_button->signal_color_set()
					.connect( sigc::mem_fun( *this,
											&vbManage::on_colour_t_changed));
			}
			else
			{
				std::cout << "No Title colour chooser handle!" << std::endl;
			}
			
			if (!colour.set(bg_title))
			{
				bg_title = (gconfclient->
							get_default_from_schema
							("/apps/vbmanage/mainwindow/colours/bg_title"))
					.get_string();
				gconfclient->set
					("/apps/vbmanage/mainwindow/colours/bg_title", bg_title);
				pTitle_colour_button->set_color(Gdk::Color(bg_title));
			}
			
			if (buttons_in_menu)
			{
				refXml->get_widget("oddRowColour", pEdit_row_odd_colour);
				pRow_colour_odd_button = Gtk::manage(new Gtk::ColorButton());
				pRow_colour_odd_button->set_title
					(Glib::ustring("Choose a colour for the odd-numbered rows' background"));
				pRow_colour_odd_box = Gtk::manage(new Gtk::HBox(false, 10));
				pRow_colour_odd_label = 
					Gtk::manage(new Gtk::Label
								(Glib::ustring("Odd rows Background ")
								 , Gtk::ALIGN_LEFT));
				pRow_colour_odd_box->pack_start(*pRow_colour_odd_button, Gtk::PACK_SHRINK);
				pRow_colour_odd_box->pack_start(*pRow_colour_odd_label, Gtk::PACK_SHRINK);
				pEdit_row_odd_colour->remove();
				pEdit_row_odd_colour->add(*pRow_colour_odd_box);
				
				pEdit_row_odd_colour->signal_activate().connect
					(sigc::mem_fun(pRow_colour_odd_button, &Gtk::ColorButton::clicked));
			}
			else
			{
				refXml->get_widget("row_colour_odd_button", pRow_colour_odd_button);
			}
			if (pRow_colour_odd_button)
			{
				pRow_colour_odd_button->set_color(Gdk::Color(bg_row_odd));
				pRow_colour_odd_button->signal_color_set()
					.connect( sigc::mem_fun( *this,
											&vbManage::on_colour_r_o_changed));
			}
			else
			{
				std::cout << "No Odd row colour chooser handle!" << std::endl;
			}
			
			if (!colour.set(bg_row_odd))
			{
				bg_row_odd = (gconfclient->
							  get_default_from_schema
							  ("/apps/vbmanage/mainwindow/colours/bg_row_odd"))
					.get_string();
				gconfclient->set
					("/apps/vbmanage/mainwindow/colours/bg_row_odd", bg_row_odd);
				pRow_colour_odd_button->set_color(Gdk::Color(bg_row_odd));
			}
			if (buttons_in_menu)
			{
				refXml->get_widget("evenRowColour", pEdit_row_even_colour);
				pRow_colour_even_button = Gtk::manage(new Gtk::ColorButton());
				
				pRow_colour_even_button->set_title
					(Glib::ustring("Choose a colour for the even-numbered rows' background"));
				pRow_colour_even_box = Gtk::manage(new Gtk::HBox(false, 10));
				pRow_colour_even_label = 
					Gtk::manage(new Gtk::Label
								(Glib::ustring("Even rows Background ")
								 , Gtk::ALIGN_LEFT));
				pRow_colour_even_box->pack_start(*pRow_colour_even_button, Gtk::PACK_SHRINK);
				pRow_colour_even_box->pack_start(*pRow_colour_even_label, Gtk::PACK_SHRINK);
				pEdit_row_even_colour->remove();
				pEdit_row_even_colour->add(*pRow_colour_even_box);
				
				pEdit_row_even_colour->signal_activate().connect
					(sigc::mem_fun(pRow_colour_even_button, &Gtk::ColorButton::clicked));
			}
			else
			{
				refXml->get_widget("row_colour_even_button", pRow_colour_even_button);
			}
			if (pRow_colour_even_button)
			{
				pRow_colour_even_button->set_color(Gdk::Color(bg_row_even));
				pRow_colour_even_button->signal_color_set()
					.connect( sigc::mem_fun( *this,
											&vbManage::on_colour_r_e_changed));
			}
			else
			{
				std::cout << "No Odd row colour chooser handle!" << std::endl;
			}
						
			if (!colour.set(bg_row_even))
			{
				bg_row_even = (gconfclient->
							   get_default_from_schema
							   ("/apps/vbmanage/mainwindow/colours/bg_row_even"))
					.get_string();
				gconfclient->set
					("/apps/vbmanage/mainwindow/colours/bg_row_even", bg_row_even);
				pRow_colour_even_button->set_color(Gdk::Color(bg_row_even));
			}
			//instantiate font chooser and validate font name 
			if (buttons_in_menu)
			{
				refXml->get_widget("font", pEdit_font);
				pFont_button = Gtk::manage(new Gtk::FontButton());
				
				pFont_button->set_title
					(Glib::ustring("Choose a font for the text in the table"));
				pFont_button->set_show_size(true);
				pFont_button->set_show_style(true);
				pFont_button->set_use_font(false);
				pFont_button->set_use_size(false);
				pFont_box = Gtk::manage(new Gtk::HBox(false, 10));
				pFont_label = 
					Gtk::manage(new Gtk::Label
								(Glib::ustring("Table Font ")
								 , Gtk::ALIGN_LEFT));
				pFont_box->pack_start(*pFont_button, Gtk::PACK_SHRINK);
				pFont_box->pack_start(*pFont_label, Gtk::PACK_SHRINK);
				pEdit_font->remove();
				pEdit_font->add(*pFont_box);
				
				pEdit_font->signal_activate().connect
					(sigc::mem_fun(pFont_button, &Gtk::FontButton::clicked));
			}
			else
			{
				refXml->get_widget("fontbutton1", pFont_button);
			}
			
			if (pFont_button)
			{
				pFont_button->set_font_name(fontname);
				pFont_button->signal_font_set()
					.connect( sigc::mem_fun( *this,
											&vbManage::on_font_selected));
			}
			else
			{
				std::cout << "No Font Chooser handle!" << std::endl;
			}
			
			Glib::RefPtr<Pango::FontMap> fm = pWindow->get_pango_context()->get_font_map();
			std::vector<Glib::RefPtr<Pango::FontFamily> > ffs = fm->list_families();
			Pango::FontDescription fd(fontname);
			
			std::vector<Glib::RefPtr<Pango::FontFamily> >::iterator ffsIt;
			bool foundfont = false;
			for (ffsIt = ffs.begin() ; ffsIt != ffs.end() ; ffsIt++)
			{
				if (fd.get_family() == (*ffsIt)->get_name())
				{
					foundfont = true;
					break;
				}
			}
			
			
			if (!foundfont)
			{
				
				fontname = (gconfclient->
							get_default_from_schema
							("/apps/vbmanage/mainwindow/font")).
					get_string();
				gconfclient->set("/apps/vbmanage/mainwindow/font", fontname);
				pFont_button->set_font_name(fontname);
				
			}
			
			pWindow->set_default_size(old_window_width, old_window_height);
			pWindow->move(old_window_xpos, old_window_ypos);
			
			connection_w = gconfclient->
				notify_add( "/apps/vbmanage/mainwindow/width",
						   sigc::mem_fun
						   (*this, &vbManage::on_window_w_changed));
			connection_h = gconfclient->
				notify_add( "/apps/vbmanage/mainwindow/height",
						   sigc::mem_fun
						   (*this, &vbManage::on_window_h_changed));
			connection_x = gconfclient->
				notify_add( "/apps/vbmanage/mainwindow/xpos",
						   sigc::mem_fun
						   (*this, &vbManage::on_window_x_changed));
			connection_y = gconfclient->
				notify_add( "/apps/vbmanage/mainwindow/ypos",
						   sigc::mem_fun
						   (*this, &vbManage::on_window_y_changed));
			connection_f = gconfclient->
				notify_add( "/apps/vbmanage/mainwindow/font",
						   sigc::mem_fun
						   (*this, &vbManage::on_font_external_changed));
			connection_bg_w = gconfclient->
				notify_add( "/apps/vbmanage/mainwindow/colours/bg_window",
						   sigc::mem_fun
						   (*this, &vbManage::on_colour_w_external_changed));
			connection_bg_t = gconfclient->
				notify_add( "/apps/vbmanage/mainwindow/colours/bg_title",
						   sigc::mem_fun
						   (*this, &vbManage::on_colour_t_external_changed));
			connection_bg_r_o = gconfclient->
				notify_add( "/apps/vbmanage/mainwindow/colours/bg_row_odd",
						   sigc::mem_fun
						   (*this, &vbManage::on_colour_r_o_external_changed));
			connection_bg_r_e = gconfclient->
				notify_add( "/apps/vbmanage/mainwindow/colours/bg_row_even",
						   sigc::mem_fun
						   (*this, &vbManage::on_colour_r_e_external_changed));
			connection_autorun = gconfclient->
				notify_add( "/apps/vbmanage/auto_run",
						   sigc::mem_fun
						   (*this, &vbManage::on_autorun_external_toggled ));
			
			
			
			refXml->get_widget("file_quit", pFile_quit);
			if (!pFile_quit)
				std::cout << "No File->Quit handle!" << std::endl;
			pFile_quit->signal_activate()
				.connect( sigc::mem_fun( *this, &vbManage::on_quit_button_clicked));
			
			
			
			refXml->get_widget("help_about", pHelp_about);
			if (!pHelp_about)
				std::cout << "No Help->About handle!" << std::endl;
			
			
			refXml->get_widget("help_help", pHelp_help);
			if (!pHelp_help)
				std::cout << "No Help->Help handle!" << std::endl;
			
			
			//~ refXml->get_widget("help_dialog", pHelp_help_dialog);
			//~ if (!pHelp_help_dialog)
			//~ std::cout << "No Help-Dialog handle!" << std::endl;
			
			//~ refXml->get_widget("about_dialog", pHelp_about_dialog);
			//~ if (!pHelp_about_dialog)
			//~ std::cout << "No About-Dialog handle!" << std::endl;
			
			pHelp_help->signal_activate()
				.connect( sigc::mem_fun( *this, &vbManage::on_help_help));
			pHelp_about->signal_activate()
				.connect( sigc::mem_fun( *this, &vbManage::on_help_about));
			
			// Make View and put it in the Scrolled Window:
			pView_event_box = Gtk::manage( new Gtk::EventBox());
			pView = Gtk::manage( new View::View(localModel));
			
			
			
			pView_event_box->modify_bg(Gtk::STATE_NORMAL, Gdk::Color(bg_window));
			//pView_event_box->modify_bg(Gtk::STATE_SELECTED, gold);
			
			pView_event_box->add(* pView);
			pScr_window->add(* pView_event_box);
			
			// Connect view's sensitivity signal to function here
			pView->sensitivity_signal().connect
				(sigc::mem_fun( *this, &vbManage::on_sensitivity_signal));
			
			// call on_timeout function now to populate Model
			vbManage::on_timeout ();
			
			// create slot for timeout signal
			sigc::slot<bool> my_slot = sigc::mem_fun(*this, &vbManage::on_timeout);
			
			// connect slot to timeout signal every 5 seconds
			sigc::connection conn = Glib::signal_timeout().connect_seconds(my_slot, 5);
			
			//connect on-idle signal to timeout slot
			//sigc::connection conn = Glib::signal_idle().connect(my_slot);
			
			// show window contents
			pWindow->show_all_children ();
		}
		catch (Gnome::Conf::Error e) { std::cout << "font validate: GConf error: " << e.code() << endl;}
		catch (...) {std::cout << "font validate: Exception!";}
		
	}
	catch (Gnome::Conf::Error e) { std::cout << "GConf error: " << e.code() << endl;}
	catch (...) {std::cout << "Exception!";}
}



vbManage::~vbManage()
{
	if (about_dialog_loaded) delete &refAboutXml;
	if (help_dialog_loaded) delete &refHelpXml;
}

bool vbManage::on_timeout()
{
	//std::cout << "This is timer " << timer_count << std::endl;
	//~ guint message_id = pStatus_bar->push("Updating information");
	//~ pProgress_bar->set_text("Updating...");
	//~ pProgress_bar->set_pulse_step(0.1);
	//~ pProgress_bar->pulse();
	
	
	try {
		
		//~ // get a set of existing row uuids:
		MModel::modelmap::iterator mrIt;
		//~ set<Data::STR> existinguuids;
		Glib::ustring methodname("in vbManage timeout loop\n");
		
		//~ for (mrIt = localModel.begin() ; 
			 //~ mrIt != localModel.end() ; mrIt++)
		//~ {
			//~ existinguuids.insert(mrIt->first);
		//~ }
		
		// make set of newly read in uuids:
		set<Data::STR> newuuids;
		
		// read in /etc/viewboxtool/machines.conf and place names and ports
		//into vectors
		vector<std::string> vbtool_names;
		vector<std::string> vbtool_ports;
		std::string mcline;
		ifstream machines_conf ("/etc/vboxtool/machines.conf", ios::in);
		if (machines_conf.is_open())
		{
			while (!machines_conf.eof() )
			{
				getline(machines_conf, mcline);
				size_t comma;
				comma = mcline.find(',');
				vbtool_names.push_back(mcline.substr(0, comma ));
				vbtool_ports.push_back(mcline.substr(comma+1));
				if (timer_count ==0)
				{
					//*cout << mcline << endl << "£" << mcline.substr(0, comma ) ;
					//*cout << "£   \t{{" <<  mcline.substr(comma+1) << "}}\n" ;
				}
			}
			machines_conf.close();
		}
		else
		{
			methodname.append("While trying to open the vboxtool machines.conf file\n");
			Glib::ustring error_text("Could not open machines.conf");
			output_dialog (methodname, error_text);
			//cout << "Could not open machines.conf" << endl;
		}
		
		//set up pipe for get vms command
		FILE * fpipe;
		std::string com ("VBoxManage --nologo list vms ;  exit");
		
		char line[256];
		std::vector<Glib::ustring> vmlines;
		std::vector<Glib::ustring>::iterator vmlIt;
		std::string lin (256,0);
		//open pipe:
		if ( !(fpipe= (FILE*)popen(com.c_str(), "r")) )
		{  // if fpipe is null
			perror("Problems with get-vms pipe");
			//std::cout << "Broken pipe" << std::endl;
		}
		// make a Model Row to put data in temporarily
		Data::STR fakeaddress= "127.0.0.1This should be a dummy";
		//ModelRow row(fakeaddress);
		
		//read in lines from pipe
		while (fgets( line, sizeof line, fpipe))
		{
			lin = line;
			vmlines.push_back(Glib::ustring(lin));
			//cout << line << endl;
		}
		timer_count++;
		pclose(fpipe);
		
		
		
		// interested in lines beginning:
		// VMState=" following characters until next " 
		// vrdp=" should say on"
		// vrdpport= should have unquoted number if so
		// vrdpports=" should have port number before next quote even if
		//not available.
		const std::string state_marker ("VMState=\"");
		const std::string vrdp_marker ("vrdp=\"");
		const std::string vrdpport_marker ("vrdpport=");
		const std::string vrdpports_marker ("vrdpports=\"");
		std::string vrdpport;
		std::string vrdpports;
		std::string vrdp;
		//cout << "Number of vmlines= "<< vmlines.size()<<endl;
		int vmcount =0;
		for (vmlIt = vmlines.begin() ; vmlIt != vmlines.end() ; vmlIt++, vmcount++)
		{
			//cout << "Row: "<< vmcount << endl;
			lin = *vmlIt;
			ModelRow row(fakeaddress);
			size_t first_dq = lin.find('"');
			size_t second_dq = lin.find('"', first_dq + 1);
			size_t open_brace = lin.find('{');
			size_t close_brace = lin.find('}');
			Glib::ustring name = lin.substr(first_dq + 1, second_dq - first_dq -1);
			Glib::ustring uuid = lin.substr(open_brace+1, close_brace - open_brace -1);
			row.set_str_value(Data::name, name);
			row.set_str_value(Data::uuid, uuid);
			newuuids.insert(uuid);
			
			//~ //if this uuid is in existinguuids, delete it.
			//~ set<Data::STR>::iterator exIdIt ;
			//~ exIdIt = existinguuids.find(uuid);
			//~ if (exIdIt != existinguuids.end() )
			//~ {
			//~ existinguuids.erase(uuid);
			//~ }
			
			
			//cout << "\tName =" << name << ".\t uuid\t"<<uuid<<endl;
			// get further information on this machine by opening second pipe:
			FILE * gpipe;
			std::string gcom ("VBoxManage --nologo showvminfo ");
			gcom.append(uuid).append(" --machinereadable  ; exit");
			//std::cout <<"<<"<< gcom <<">>"<< std::endl;
			char gline[256];
			std::string glin (256,0);
			methodname = Glib::ustring("on timeout inner loop");
			Glib::ustring lines;
			//open second pipe for info on each machine
			if ( !(gpipe= (FILE*)popen(gcom.c_str(), "r")) )
			{  // if fpipe is null
				perror("Problems with inner pipe");
				Glib::ustring error_text("Problems with inner pipe");
				output_dialog (methodname, error_text);
				//std::cout << "Broken inner pipe" << std::endl;
			}
			
			// search through lines of input for relevant information
			
			
			//~ std::cout <<"<<"<< state_marker <<">>"<< std::endl;
			//~ std::cout <<"<<"<< vrdp_marker <<">>"<< std::endl;
			//~ std::cout <<"<<"<< vrdpport_marker <<">>"<< std::endl;
			//~ std::cout <<"<<"<< vrdpports_marker <<">>"<< std::endl;
			
			
			//pProgress_bar->set_pulse_step(0.3);
			// get lines of input from second pipe
			while (fgets( gline, sizeof gline, gpipe))
			{
				//~ pProgress_bar->pulse();
				//~ pProgress_bar->show();
				//~ pWindow->show_all_children ();
				glin = gline;
				lines.append(gline, glin.size());
				lines.append("\n");
			}
			
			pclose(gpipe);
			if ( lines.find("ERROR") != std::string::npos)
			{
				output_dialog(methodname, lines);
			}
			while (!lines.empty())
			{
				size_t endline = lines.find('\n');
				glin = lines.substr(0 , endline);
				lines.erase(0 , endline+1);
				//*cout << gline;
				// if this is the State= line
				if (glin.substr(0,state_marker.size()).compare(state_marker) == 0 )
				{
					//	std::cout << glin << std::endl;
					size_t state_end = glin.substr(state_marker.size())
						.find_first_of('"')  ;
					std::string state = glin.substr(state_marker.size(), state_end);
					//*cout << "\t\tState:" << state << ".\n";
					// set row's state to display state corredponding to read-in state
					row.set_str_value(Data::status, display_states[states[state]]);
					
					//std::cout <<"<<"<< state <<">>"<< std::endl;
					// set bools in row according to the read-in state	
					switch (states[state])
					{
						case 0: 
						//"running":
						{
							row.set_boo_value(Data::start, false);
							row.set_boo_value(Data::acpi, true);
							row.set_boo_value(Data::kill, true);
							row.set_boo_value(Data::save, true);
							row.set_boo_value(Data::resume, false);
							row.set_boo_value(Data::pause, true);
						}
							break;
						case 1: 
						//"saved":
						{
							row.set_boo_value(Data::start, true);
							row.set_boo_value(Data::acpi, false);
							row.set_boo_value(Data::kill, false);
							row.set_boo_value(Data::save, false);
							row.set_boo_value(Data::resume, false);
							row.set_boo_value(Data::pause, false);
						}
							break;
						case 2: 
						//"poweroff":
						{
							row.set_boo_value(Data::start, true);
							row.set_boo_value(Data::acpi, false);
							row.set_boo_value(Data::kill, false);
							row.set_boo_value(Data::save, false);
							row.set_boo_value(Data::resume, false);
							row.set_boo_value(Data::pause, false);
						}
							break;
						case 3: 
						//"aborted":
						{
							row.set_boo_value(Data::start, false);
							row.set_boo_value(Data::acpi, false);
							row.set_boo_value(Data::kill, true);
							row.set_boo_value(Data::save, false);
							row.set_boo_value(Data::resume, false);
							row.set_boo_value(Data::pause, true);
						}
							break;
						case 4: 
						//"paused":
						{
							row.set_boo_value(Data::start, false);
							row.set_boo_value(Data::acpi, false);
							row.set_boo_value(Data::kill, false);
							row.set_boo_value(Data::save, false);
							row.set_boo_value(Data::resume, true);
							row.set_boo_value(Data::pause, false);
						}
							break;
						case 5: 
						//"unknown":
						{
							row.set_boo_value(Data::start, false);
							row.set_boo_value(Data::acpi, false);
							row.set_boo_value(Data::kill, false);
							row.set_boo_value(Data::save, false);
							row.set_boo_value(Data::resume, false);
							row.set_boo_value(Data::pause, false);
						}
							break;
						default:
							std::cout << "default reached in switch(state) ! " << 
								std::endl;
					}
					
					
				}
				// if this line is the vrdpport line
				
				else if (glin.substr(0,vrdpport_marker.size()).compare(
																	   vrdpport_marker) == 0 )
				{
					size_t vrdpport_end = glin.substr(vrdpport_marker.size())
						.find_first_of('\n')  ;
					vrdpport = glin.substr(
										   vrdpport_marker.size(), vrdpport_end);
					//*cout << "\t\tvrdpport:" << vrdpport << "." << endl;
				}
				//if this line is the vrdpports line
				else if (glin.substr(0,vrdpports_marker.size()).compare(
																		vrdpports_marker) == 0 )
				{
					size_t vrdpports_end = glin.substr(vrdpports_marker.size())
						.find_first_of('"')  ;
					vrdpports = glin.substr(
											vrdpports_marker.size(), vrdpports_end);
					//*cout << "\t\tvrdpports:" << vrdpports << "."<< endl;
				}
				// if this line is the VRDP status line
				else if (glin.substr(0,vrdp_marker.size()).compare(
																   vrdp_marker) == 0 )
				{
					size_t vrdp_end = glin.substr(vrdp_marker.size())
						.find_first_of('"')  ;
					vrdp = glin.substr(vrdp_marker.size(), vrdp_end);
					//*cout << "\t\tvrdp:" << vrdp<< "."<< endl;
				}
			} //end of second line loop
			// Console is possible iff vrdp = "on" && ( vrdpport is a number
			// || vrdpports is a number)
			const std::string numbers("1234567890");
			size_t found;
			found = vrdpport.find_first_not_of(numbers);
			if ( (vrdpport.size() == 0) || (found != string::npos) )
			{
				//vrdpport contains non-numerals or is empty: try vrdpports
				found = vrdpports.find_first_not_of(numbers);
				if ( (vrdpports.size() == 0) || (found != string::npos))
				{
					// no portnumber to be found
					//*cout << "Can't find portnumber\n";
					row.set_str_value(Data::port, "none");
					row.set_boo_value(Data::console, false);
				}
				else
				{
					// vrdpports is ok
					if ( vrdp.compare("on") == 0)
					{
						row.set_str_value(Data::port, vrdpports);
						row.set_boo_value(Data::console, true);
						//*cout << "vrdpports ok, vrdp on\n";
					}
					else
					{
						row.set_str_value(Data::port, "off");
						row.set_boo_value(Data::console, false);
						//*cout << "vrdpports ok, vrdp off\n";
					}
				}
			}
			else
			{
				//vrdpport is ok
				if ( vrdp.compare("on") == 0)
				{
					row.set_str_value(Data::port, vrdpport);
					row.set_boo_value(Data::console, true);
					//*cout << "vrdpport ok, vrdp on\n";
				}
				else
				{
					row.set_str_value(Data::port, "off");
					row.set_boo_value(Data::console, false);
					//*cout << "vrdpport ok, vrdp off\n";
				}
			}
			// fill in autostart and autostartstatus
			vector<std::string>::iterator vbt_n_it;
			vector<std::string>::iterator vbt_p_it;
			bool vbtool_name_found = false;
			for (vbt_n_it = vbtool_names.begin(), vbt_p_it = vbtool_ports.begin() ;
				 (vbt_n_it != vbtool_names.end() && vbt_p_it != vbtool_ports.end()); 
				 vbt_n_it++, vbt_p_it++)
			{
				if ((*vbt_n_it) == row.get_str_value(Data::name) )
				{
					vbtool_name_found = true;
					row.set_boo_value(Data::autostart,true);
					if ( (*vbt_p_it) != row.get_str_value(Data::port))
					{
						row.set_str_value(Data::autostartstatus,
										  "Wrong port set: " + (*vbt_p_it));
					}
					else
					{
						row.set_str_value(Data::autostartstatus, "");
					}
					break;
				}
			}
			if (!vbtool_name_found )
			{
				row.set_boo_value(Data::autostart,false);
				row.set_str_value(Data::autostartstatus, ""/*"Not found"*/);
			}
			
			// row is now full of data. Check if localModel has row with same uuid
			// if not, add row to localModel. If so, check for differences and update.
			bool found_existing = false;
			mrIt = localModel.find(row.get_str_value(Data::uuid));
			//~ for (mrIt = localModel.begin() ; 
			//~ mrIt != localModel.end() ; mrIt++)
			//~ {
			if ( mrIt != localModel.end() )
			{
				found_existing = true;
				for ( int count = 0 ; count != Data::n_m_s_cols ; count++ )
				{
					mrIt->second.set_str_value(Data::m_s_col(count),
											   row.get_str_value(Data::m_s_col(count)));
				}
				for ( int count = 0 ; count != Data::n_m_b_cols ; count++ )
				{
					mrIt->second.set_boo_value(Data::m_b_col(count),
											   row.get_boo_value(Data::m_b_col(count)));
				}
				
			}
			//~ }
			else
			{
				//	cout << "Adding row..."<< name << endl;
				localModel.insert(row);
			}
			
			
			//cout << "after row "<<vmcount << " vmlines.size()="<< vmlines.size() <<endl;	
			
		}
		//pStatus_bar->pop();
		//	pProgress_bar->set_text("");
		//pProgress_bar->set_fraction(0.0);
		//		cout << "That was timer " << timer_count <<endl;
		
		//~ // Now if there are any existinguuids left, delete the relevant modelrow.
		//~ if (!existinguuids.empty())
		//~ {
			//~ //cout << "Erasing rows..." << endl;	
			//~ std::set<Data::STR>::iterator delIt;
			//~ for (delIt = existinguuids.begin() ; delIt != existinguuids.end() ; delIt++)
			//~ {
				//~ MModel::modelmap::iterator findIt = localModel.find(*delIt);
				//~ if (findIt != localModel.end())
				//~ {
					//~ localModel.erase(localModel.find((*delIt)));
				//~ }
				//~ //cout<< "Erasing " << *delIt << endl;
			//~ }
		//~ }
		
		// Now if there are any rows in localModel not in newuuids, order their deletion
		std::set<Data::STR> uuidsToDelete;
		for (mrIt = localModel.begin() ; mrIt != localModel.end() ; mrIt++)
		{
			std::set<Data::STR>::iterator keepIt;
			keepIt = newuuids.find(mrIt->first);
			if (keepIt == newuuids.end() )
			{
				uuidsToDelete.insert(mrIt->first);
			}
		}
		while (!uuidsToDelete.empty() )
		{
			localModel.erase(localModel.find((*uuidsToDelete.begin())));
			uuidsToDelete.erase(uuidsToDelete.begin());
		}
	}
	catch (...)
	{ std::cout << "Some exception in on_timeout" << std::endl;
	}
	//pStatus_bar->remove_message(message_id);
	return true;
}

void vbManage::on_quit_button_clicked()
{
	try {
		
		pWindow->hide();
		
	}
	catch (...) 
	{ std::cout << "default exception!" <<std::endl;
	}
}


void vbManage::on_help_about ()
{
	if (!about_dialog_loaded)
	{
		try
		{
			refAboutXml = Gnome::Glade::Xml::create(GLADE_FILE_ABOUT_DIALOG);
		}
		catch(const Gnome::Glade::XmlError& ex)
		{
			std::cerr << ex.what() << std::endl;
		}
		refAboutXml->get_widget("about_dialog", pHelp_about_dialog);
		about_dialog_loaded = true;
	}
	int response = pHelp_about_dialog->run();
	//	std::cout << "About dialog response: "<< response << endl;
	pHelp_about_dialog->hide();
	//pHelp_about_dialog->show();
}

void vbManage::on_help_help ()
{
	if (!help_dialog_loaded)
	{
		try
		{
			refHelpXml = Gnome::Glade::Xml::create(GLADE_FILE_HELP_DIALOG);
		}
		catch(const Gnome::Glade::XmlError& ex)
		{
			std::cerr << ex.what() << std::endl;
		}
		refHelpXml->get_widget("help_dialog", pHelp_help_dialog);
		help_dialog_loaded = true;
	}
	int response = pHelp_help_dialog->run();
	//	std::cout << "Help Dialog response: " << response << endl;
	pHelp_help_dialog->hide();
	//pHelp_help_dialog->show();
}

//~ void vbManage::on_edit_font ()
//~ {
	//~ if (!font_dialog_loaded)
	//~ {
		//~ try
		//~ {
			//~ refFontXml = Gnome::Glade::Xml::create(GLADE_EDIT_FONT_DIALOG);
		//~ }
		//~ catch(const Gnome::Glade::XmlError& ex)
		//~ {
			//~ std::cerr << ex.what() << std::endl;
		//~ }
		//~ refFontXml->get_widget("fontselectiondialog1", pEdit_font);
		//~ font_dialog_loaded = true;
	//~ }
	//~ pEdit_font->set_title(Glib::ustring("Set font"));
	//~ int response = pEdit_font->run();
	//~ if (response = Gtk::RESPONSE_OK)
	//~ {
		
	//~ }
	//~ //	std::cout << "About dialog response: "<< response << endl;
	//~ pHelp_about_dialog->hide();
	//~ //pHelp_about_dialog->show();
//~ }


void vbManage::on_sensitivity_signal(std::vector<bool> sens)
{
	//~ pButton_all_off->property_sensitive().set_value(sens[0]);
	//~ pButton_all_acpi->property_sensitive().set_value(sens[1]);
	//~ pButton_all_save->property_sensitive().set_value(sens[2]);
	//~ pButton_all_pause->property_sensitive().set_value(sens[3]);
	//~ pButton_all_resume->property_sensitive().set_value(sens[4]);
	//~ pButton_all_start->property_sensitive().set_value(sens[5]);
	//~ pButton_all_console->property_sensitive().set_value(sens[6]);
}


void vbManage::on_all_off_clicked()
{
	std::vector<ViewRow::ViewRow *>::iterator rowIt;
	for (rowIt = pView->rows.begin() ; rowIt != pView->rows.end() ; rowIt++)
	{
		if ( (*rowIt)->hardoff.property_sensitive().get_value() )
		{
			(*rowIt)->on_hardoff_button_clicked();
		}
	}
}


void vbManage::on_all_acpi_clicked()
{
	std::vector<ViewRow::ViewRow *>::iterator rowIt;
	for (rowIt = pView->rows.begin() ; rowIt != pView->rows.end() ; rowIt++)
	{
		if ( (*rowIt)->softoff.property_sensitive().get_value() )
		{
			(*rowIt)->on_softoff_button_clicked();
		}
	}
}


void vbManage::on_all_save_clicked()
{
	std::vector<ViewRow::ViewRow *>::iterator rowIt;
	for (rowIt = pView->rows.begin() ; rowIt != pView->rows.end() ; rowIt++)
	{
		if ( (*rowIt)->save.property_sensitive().get_value() )
		{
			(*rowIt)->on_save_button_clicked();
		}
	}
}


void vbManage::on_all_pause_clicked()
{
	std::vector<ViewRow::ViewRow *>::iterator rowIt;
	for (rowIt = pView->rows.begin() ; rowIt != pView->rows.end() ; rowIt++)
	{
		if ( (*rowIt)->pause.property_sensitive().get_value() )
		{
			(*rowIt)->on_pause_button_clicked();
		}
	}
}


void vbManage::on_all_resume_clicked()
{
	//~ std::vector<ViewRow::ViewRow *>::iterator rowIt;
	//~ for (rowIt = pView->rows.begin() ; rowIt != pView->rows.end() ; rowIt++)
	//~ {
	//~ if ( (*rowIt)->resume.property_sensitive().get_value() )
	//~ {
	//~ (*rowIt)->on_resume_button_clicked();
	//~ }
	//~ }
}


void vbManage::on_all_start_clicked()
{
	std::vector<ViewRow::ViewRow *>::iterator rowIt;
	for (rowIt = pView->rows.begin() ; rowIt != pView->rows.end() ; rowIt++)
	{
		if ( (*rowIt)->start.property_sensitive().get_value() )
		{
			(*rowIt)->on_run_button_clicked();
		}
	}
}


void vbManage::on_all_console_clicked()
{
	std::vector<ViewRow::ViewRow *>::iterator rowIt;
	for (rowIt = pView->rows.begin() ; rowIt != pView->rows.end() ; rowIt++)
	{
		if ( (*rowIt)->console.property_sensitive().get_value() )
		{
			(*rowIt)->on_console_button_clicked();
		}
	}
}

void vbManage::on_font_selected()
{
	//cout << pFont_button->get_font_name() << " font selected." << endl;
	fontname = pFont_button->get_font_name();
	gconfclient->set("/apps/vbmanage/mainwindow/font",fontname);
	pView->on_font_change();
}

void vbManage::on_colour_w_changed()
{
	Gdk::Color gdkCol = pWindow_colour_button->get_color();
	Pango::Color panCol ;
	panCol.set_red(gdkCol.get_red());
	panCol.set_green(gdkCol.get_green());
	panCol.set_blue(gdkCol.get_blue());
	bg_window = panCol.to_string();
	gconfclient->set("/apps/vbmanage/mainwindow/colours/bg_window", bg_window);
	pView_event_box->modify_bg(Gtk::STATE_NORMAL, Gdk::Color(bg_window));
}

void vbManage::on_colour_t_changed()
{
	Gdk::Color gdkCol = pTitle_colour_button->get_color();
	Pango::Color panCol ;
	panCol.set_red(gdkCol.get_red());
	panCol.set_green(gdkCol.get_green());
	panCol.set_blue(gdkCol.get_blue());
	bg_title = panCol.to_string();
	gconfclient->set("/apps/vbmanage/mainwindow/colours/bg_title", bg_title);
	pView->on_bg_title_change();
}

void vbManage::on_colour_r_o_changed()
{
	Gdk::Color gdkCol = pRow_colour_odd_button->get_color();
	Pango::Color panCol ;
	panCol.set_red(gdkCol.get_red());
	panCol.set_green(gdkCol.get_green());
	panCol.set_blue(gdkCol.get_blue());
	bg_row_odd = panCol.to_string();
	gconfclient->set("/apps/vbmanage/mainwindow/colours/bg_row_odd", bg_row_odd);
	pView->on_bg_row_odd_change();
}

void vbManage::on_colour_r_e_changed()
{
	Gdk::Color gdkCol = pRow_colour_even_button->get_color();
	Pango::Color panCol ;
	panCol.set_red(gdkCol.get_red());
	panCol.set_green(gdkCol.get_green());
	panCol.set_blue(gdkCol.get_blue());
	bg_row_even = panCol.to_string();
	gconfclient->set("/apps/vbmanage/mainwindow/colours/bg_row_even", bg_row_even);
	pView->on_bg_row_even_change();
}



void vbManage::output_dialog(
								 Glib::ustring & main_message, 
								 Glib::ustring & secondary_message)
{
	if (show_error_messages)
	{
		Gtk::MessageDialog dialog( main_message,
		false, Gtk::MESSAGE_WARNING, Gtk::BUTTONS_OK);
		dialog.set_secondary_text(secondary_message);

		int result = dialog.run();

		//Handle the response:
		switch(result)
		{
			case(Gtk::RESPONSE_OK):
			{
				//std::cout << "OK clicked." << std::endl;
				break;
			}
			case(Gtk::RESPONSE_CLOSE):
			{
				//std::cout << "Close button clicked." << std::endl;
				break;
			}
			default:
			{
				//std::cout << "Unexpected button clicked." << std::endl;
				break;
			}
		}
	}
}

void vbManage::on_window_x_changed(
									   guint connectionid, 
									   const Gnome::Conf::Entry& entry)
{
	try 
	{	
		if (connectionid != connection_x)
		{
			cout << "window_x: connection mismatch. connectionid = " <<
				connectionid << ", connection_x = " << connection_x << endl;
		}
		else
		{
			if (entry.get_value().get_type() == Gnome::Conf::VALUE_INT)
			{
				gint value = entry.get_value().get_int();
				if ((value >= -1) && 
					(value < gscreen->get_width() -
					 gconfclient->get("/apps/vbmanage/mainwindow/width").get_int()))
				{
					gint y;
					Gnome::Conf::Entry yentry = gconfclient
						->get_entry("/apps/vbmanage/mainwindow/ypos");
					if (yentry.get_value().get_type() == Gnome::Conf::VALUE_INT)
					{
						y = yentry.get_value().get_int();
					}
					else
					{
						y = old_window_ypos;
						cout << "In window_x, y entry not an int" << endl;
					}
					pWindow->move(value, y);
					old_window_xpos = value;
				}
				else
				{
					Glib::ustring first("Changing x-position of main window from external source");
					Glib::ustring second("An out of range value for main window x-position has been entered, ");
					second.append("externally to this program. Ignoring it.");
					output_dialog(first, second);
				}
			}
		}
	}
	catch (Gnome::Conf::Error e) { std::cout << "Window_x: GConf error: " << e.code() << endl;}
	catch (...) {std::cout << "Window x: Exception!";}
}

void vbManage::on_window_y_changed(
									   guint connectionid, 
									   const Gnome::Conf::Entry& entry)
{
	try
	{	
		if (connectionid != connection_y)
		{
			cout << "window_y: connection mismatch. connectionid = " <<
				connectionid << ", connection_y = " << connection_y << endl;
		}
		else
		{
			if (entry.get_value().get_type() == Gnome::Conf::VALUE_INT)
			{
				gint value = entry.get_value().get_int();
				if ((value >= -1) && 
					(value < gscreen->get_height() -
					 gconfclient->get("/apps/vbmanage/mainwindow/height").get_int()))
				{
					gint x;
					Gnome::Conf::Entry xentry = gconfclient
						->get_entry("/apps/vbmanage/mainwindow/xpos");
					if (xentry.get_value().get_type() == Gnome::Conf::VALUE_INT)
					{
						x = xentry.get_value().get_int();
					}
					else
					{
						x = old_window_xpos;
						cout << "In window_y, x entry not an int" << endl;
					}
					pWindow->move(x, value);
					old_window_ypos = value;
				}
				else
				{
					Glib::ustring first("Changing y-position of main window from external source");
					Glib::ustring second("An out of range value for main window y-position has been entered, ");
					second.append("externally to this program. Ignoring it.");
					output_dialog(first, second);
				}
			}
		}
	}
	catch (Gnome::Conf::Error e) { std::cout << "Window y: GConf error: " << e.code() << endl;}
	catch (...) {std::cout << "Window Y: Exception!";}
}

void vbManage::on_window_w_changed(
									   guint connectionid, 
									   const Gnome::Conf::Entry& entry)
{
	try
	{	
		if (connectionid != connection_w)
		{
			cout << "window_w: connection mismatch. connectionid = " <<
				connectionid << ", connection_w = " << connection_w << endl;
		}
		else
		{
			if (entry.get_value().get_type() == Gnome::Conf::VALUE_INT)
			{
				gint value = entry.get_value().get_int();
				if ((value >= -1) && (value < gscreen->get_width() - old_window_xpos))
				{
					gint height;
					Gnome::Conf::Entry hentry = gconfclient
						->get_entry("/apps/vbmanage/mainwindow/height");
					if (hentry.get_value().get_type() == Gnome::Conf::VALUE_INT)
					{
						height = hentry.get_value().get_int();
					}
					else
					{
						height = old_window_height;
						cout << "In window_w, height entry not an int" << endl;
					}
					pWindow->resize(value, height);
					old_window_width = value;
				}
				else
				{
					Glib::ustring first("Changing width of main window from external source");
					Glib::ustring second("An out of range value for main window width has been entered, ");
					second.append("externally to this program. Ignoring it.");
					output_dialog(first, second);
				}
			}
		}
	}
	catch (Gnome::Conf::Error e) { std::cout << "Window W: GConf error: " << e.code() << endl;}
	catch (...) {std::cout << "Window W: Exception!";}
}

void vbManage::on_window_h_changed(
									   guint connectionid, 
									   const Gnome::Conf::Entry& entry)
{
	try
	{	
		if (connectionid != connection_h)
		{
			cout << "window_h: connection mismatch. connectionid = " <<
				connectionid << ", connection_h = " << connection_h << endl;
		}
		else
		{
			if (entry.get_value().get_type() == Gnome::Conf::VALUE_INT)
			{
				gint value = entry.get_value().get_int();
				if ((value >= -1) && (value < gscreen->get_height() - old_window_ypos))
				{
					gint width;
					Gnome::Conf::Entry wentry = gconfclient
						->get_entry("/apps/vbmanage/mainwindow/ypos");
					if (wentry.get_value().get_type() == Gnome::Conf::VALUE_INT)
					{
						width = wentry.get_value().get_int();
					}
					else
					{
						width = old_window_width;
						cout << "In window_h, width entry not an int" << endl;
					}
					pWindow->resize(width, value);
					old_window_height = value;
				}
				else
				{
					Glib::ustring first("Changing height of main window from external source");
					Glib::ustring second("An out of range value for main window height has been entered, ");
					second.append("externally to this program. Ignoring it.");
					output_dialog(first, second);
				}
			}
		}
	}
	catch (Gnome::Conf::Error e) { std::cout << "Window H: GConf error: " << e.code() << endl;}
	catch (...) {std::cout << "Window H: Exception!";}
}

void vbManage::on_font_external_changed(
											guint connectionid, 
											const Gnome::Conf::Entry& entry)
{
	if (entry.get_value().get_type() == Gnome::Conf::VALUE_STRING)
	{
		Glib::ustring value = entry.get_value().get_string();
		if (value != fontname)
		{
			// validate value
			Glib::RefPtr<Pango::FontMap> fm = pWindow->get_pango_context()->get_font_map();
			std::vector<Glib::RefPtr<Pango::FontFamily> > ffs = fm->list_families();
			Pango::FontDescription fd(fontname);
			
			std::vector<Glib::RefPtr<Pango::FontFamily> >::iterator ffsIt;
			bool foundfont = false;
			for (ffsIt = ffs.begin() ; ffsIt != ffs.end() ; ffsIt++)
			{
				if (fd.get_family() == (*ffsIt)->get_name())
				{
					foundfont = true;
					break;
				}
			}
			
			
			if (foundfont)
			{
				// set fontname
				pFont_button->set_font_name(value);
				fontname = value;
				// emit signal to view
				on_font_selected();
			}
		}
	}
}

void vbManage::on_colour_w_external_changed(
												guint connectionid, 
												const Gnome::Conf::Entry& entry)
{
	if (entry.get_value().get_type() == Gnome::Conf::VALUE_STRING)
	{
		Glib::ustring value = entry.get_value().get_string();
		if (value != bg_window)
		{
			// validate value
			
			Gdk::Color colour;
			if (colour.set(value))
			{
				// set bg_window
				pWindow_colour_button->set_color(colour);
				bg_window = value;
				// emit signal to view
				on_colour_w_changed();
			} else cout<< "Invalid colour!"<<endl;
		}
	}
}

void vbManage::on_colour_t_external_changed(
												guint connectionid, 
												const Gnome::Conf::Entry& entry)
{
	if (entry.get_value().get_type() == Gnome::Conf::VALUE_STRING)
	{
		Glib::ustring value = entry.get_value().get_string();
		if (value != bg_title)
		{
			// validate value
			
			Gdk::Color colour;
			if (colour.set(value))
			{
				// set bg_title
				pTitle_colour_button->set_color(colour);
				bg_title = value;
				// emit signal to view
				on_colour_t_changed();
			}else cout<< "Invalid colour!"<<endl;
		}
	}
}

void vbManage::on_colour_r_o_external_changed(
												  guint connectionid, 
												  const Gnome::Conf::Entry& entry)
{
	if (entry.get_value().get_type() == Gnome::Conf::VALUE_STRING)
	{
		Glib::ustring value = entry.get_value().get_string();
		if (value != bg_row_odd)
		{
			// validate value
			
			Gdk::Color colour;
			if (colour.set(value))
			{
				// set bg_title
				pRow_colour_odd_button->set_color(colour);
				bg_row_odd = value;
				// emit signal to view
				on_colour_r_o_changed();
			}else cout<< "Invalid colour!"<<endl;
		}
	}
}

void vbManage::on_colour_r_e_external_changed(
												  guint connectionid, 
												  const Gnome::Conf::Entry& entry)
{
	if (entry.get_value().get_type() == Gnome::Conf::VALUE_STRING)
	{
		Glib::ustring value = entry.get_value().get_string();
		if (value != bg_row_even)
		{
			// validate value
			
			Gdk::Color colour;
			if (colour.set(value))
			{
				// set bg_title
				pRow_colour_even_button->set_color(colour);
				bg_row_even = value;
				// emit signal to view
				on_colour_r_e_changed();
			}else cout<< "Invalid colour!"<<endl;
		}
	}
}

void vbManage::on_autorun_toggled()
{
	autorun = pFile_autorun->get_active();
	gconfclient->set("/apps/vbmanage/auto_run", autorun);
	make_autorun_file();
}

void vbManage::on_autorun_external_toggled(
											   guint connectionid, 
											   const Gnome::Conf::Entry& entry)
{
	if (entry.get_value().get_type() == Gnome::Conf::VALUE_BOOL)
	{
		bool value = entry.get_value().get_bool();
		if (value != autorun)
		{
			
			// set global variable
			pFile_autorun->set_active(value);
			autorun = value;
			make_autorun_file();			
		}
	}
}


void vbManage::make_autorun_file()
{
	// first get home folder
	Glib::ustring homedir(getenv("HOME"));
	Glib::ustring sourcefile("/usr/share/applications/vbmanage.desktop ");
	Glib::ustring startfile("/.config/autostart/vbmanage.desktop ");
	Glib::ustring methodname("vbManage::make_autorun_file()");
	Glib::ustring doxology("2>/dev/null ) || echo ping : ");
	
	std::string command;
	
	// then make command depending on whether autorun is set
	if (autorun)
	{
		command = "(mkdir -p " + homedir + "/.config/autostart 2>/dev/null && ln -sv " + sourcefile + homedir + startfile + doxology;
	}
	else
	{
		command = "(rm -v " + homedir + startfile + doxology;
	}
	// then run the command
	// set up pipe
	FILE * fpipe;
	
	
	char line[256];
	Glib::ustring lines("Trying to change autostart file");
	lines.append("\n");
	lines.append(command);
	lines.append("\n");
	lines.append("response was\n\n");
	size_t preamble = lines.size();
	std::string lin (256,0);
	
	
	
	//open pipe:
	if ( !(fpipe= (FILE*)popen(command.c_str(), "r")) )
	{  // if fpipe is null
		perror("Problems with pipe");
		std::cout << "Broken pipe" << std::endl;
	}
	while (fgets( line, sizeof line, fpipe))
	{
		lin = line;
		lines.append(line, lin.size());
		lines.append("\n");
		//			cout << line << endl;
	}
	// check for errors
	// parse returned text. open dialog if not ok.
	bool response_ok = false;
	// parse lines here
	Glib::ustring unexpected(": ");
	
	if (lines.substr(preamble).find(unexpected) == Glib::ustring::npos)
	{
		response_ok = true;
	}
	// following line for testing
	// response_ok = true;
	if (!response_ok)
	{
		output_dialog(methodname, lines);
	}
	pclose(fpipe);
	
	
	
	
}

void vbManage::on_show_error_messages_toggled()
{
	show_error_messages = pFile_show_error_messages->get_active();
	gconfclient->set("/apps/vbmanage/show_error_messages", show_error_messages);
}

void vbManage::on_show_error_messages_external_toggled(
											   guint connectionid, 
											   const Gnome::Conf::Entry& entry)
{
	if (entry.get_value().get_type() == Gnome::Conf::VALUE_BOOL)
	{
		bool value = entry.get_value().get_bool();
		if (value != show_error_messages)
		{
			
			// set global variable
			pFile_show_error_messages->set_active(value);
			show_error_messages = value;	
		}
	}
}


Glib::ustring vbManage::make_markup(Glib::ustring text)
{
	return Glib::ustring(opening + fontname + middle + text + closing);
}

// static values

const std::string vbManage::display_states [6] = {"Running ",
	"Saved ",
	"Powered off ",
	"Aborted ",
	"Paused ",
	"Unknown "};

