/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * trunk
 * Copyright (C) Eoin McMahon 2010 <eoinmcmahon0@AT@gmail.com>
	 * 
 * trunk is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
	 * 
 * trunk is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _VB_MANAGE_HPP_
#define _VB_MANAGE_HPP_

// Uncomment the following line for installing program
#define FOR_INSTALLATION
#ifdef FOR_INSTALLATION
#define GLADE_FILE_WINDOW PACKAGE_DATA_DIR"/vbmanage/glade/vbmanage.glade"
#define GLADE_FILE_HELP_DIALOG PACKAGE_DATA_DIR"/vbmanage/glade/help.glade"
#define GLADE_FILE_ABOUT_DIALOG PACKAGE_DATA_DIR"/vbmanage/glade/about.glade"
#define GLADE_EDIT_FONT_DIALOG PACKAGE_DATA_DIR"/vbmanage/glade/font.glade"
#define GLADE_EDIT_COLOUR_DIALOG PACKAGE_DATA_DIR"/vbmanage/glade/colour.glade"
#else
#define GLADE_FILE_WINDOW "src/vbmanage.glade"
#define GLADE_FILE_HELP_DIALOG "src/help.glade"
#define GLADE_FILE_ABOUT_DIALOG "src/about.glade"
#define GLADE_EDIT_FONT_DIALOG "src/font.glade"
#define GLADE_EDIT_COLOUR_DIALOG "src/colour.glade"
#endif /* FOR_INSTALLATION */

#include <glib.h>
#include <iostream>
#include <vector>
#include <set>
#include <map>
#include <list>
#include <fstream>
#include <time.h>
#include <gtkmm.h>
#include <libglademm/xml.h>
#include <typeinfo>
#include <gconfmm.h>


extern Glib::RefPtr<Gnome::Conf::Client> gconfclient;
extern Glib::RefPtr<Gdk::Screen> gscreen;
extern Glib::ustring fontname;
extern Glib::ustring bg_window;
extern Glib::ustring bg_title;
extern Glib::ustring bg_row_odd;
extern Glib::ustring bg_row_even;
extern const Glib::ustring opening;
extern const Glib::ustring middle;
extern const Glib::ustring closing;
extern const int hpadding;
extern bool autorun;
extern bool show_error_messages;
#include "derived-window.hpp"
#include "m-model.hpp"
#include "view.hpp"

class vbManage
{
	public:
		static const std::string display_states [6]; 
		
DerivedWindow::DerivedWindow * pWindow;
		//vbManage(Gtk::Allocation a);
		//vbManage(Glib::RefPtr<Gnome::Conf::Client> client);
		vbManage();
		virtual ~vbManage();
		Glib::RefPtr<Gnome::Conf::Client> gconf;
		Glib::RefPtr<Gnome::Glade::Xml> refXml;
		Glib::RefPtr<Gnome::Glade::Xml> refAboutXml;
		Glib::RefPtr<Gnome::Glade::Xml> refHelpXml;
		Glib::RefPtr<Gnome::Glade::Xml> refFontXml;
		Glib::RefPtr<Gnome::Glade::Xml> refColourXml;
		bool about_dialog_loaded;
		bool help_dialog_loaded;
		bool font_dialog_loaded;
		bool colour_dialog_loaded;
		
		//Gtk::Window * pWindow;
		
		
		
		Gtk::TreeView * pTree_view;
		Gtk::Button* pQuit_button;
		Gtk::ProgressBar * pProgress_bar;
		Gtk::Statusbar * pStatus_bar;
		Gtk::ScrolledWindow * pScr_window;
		MModel::MModel localModel;
		Gtk::EventBox * pView_event_box;
		View::View * pView;
		Gtk::Button * pButton_all_off;
		Gtk::Button * pButton_all_acpi;
		Gtk::Button * pButton_all_save;
		Gtk::Button * pButton_all_pause;
		Gtk::Button * pButton_all_resume;
		Gtk::Button * pButton_all_start;
		Gtk::Button * pButton_all_console;
		Gtk::MenuBar * pMenu_bar;
		Gtk::Menu * pFile;
		Gtk::Menu * pHelp;
		Gtk::ImageMenuItem * pFile_quit;
		Gtk::ImageMenuItem * pHelp_help;
		Gtk::ImageMenuItem * pHelp_about;
		Gtk::CheckMenuItem * pFile_autorun;
		Gtk::CheckMenuItem * pFile_show_error_messages;
	
		Gtk::Dialog * pHelp_help_dialog;
		Gtk::AboutDialog * pHelp_about_dialog;
		Gtk::MenuItem * pEdit_window_colour;
		Gtk::MenuItem * pEdit_title_colour;
		Gtk::MenuItem * pEdit_row_odd_colour;
		Gtk::MenuItem * pEdit_row_even_colour;
		Gtk::MenuItem * pEdit_font;
		
		Gtk::FontButton * pFont_button;
		Gtk::ColorButton * pWindow_colour_button;
		Gtk::ColorButton * pTitle_colour_button;
		Gtk::ColorButton * pRow_colour_odd_button;
		Gtk::ColorButton * pRow_colour_even_button;
		
		Gtk::HBox * pWindow_colour_box;
		Gtk::Label * pWindow_colour_label;
		Gtk::HBox * pTitle_colour_box;
		Gtk::Label * pTitle_colour_label;
		Gtk::HBox * pRow_colour_odd_box;
		Gtk::Label * pRow_colour_odd_label;
		Gtk::HBox * pRow_colour_even_box;
		Gtk::Label * pRow_colour_even_label;
		Gtk::HBox * pFont_box;
		Gtk::Label * pFont_label;
		
		void output_dialog(
							Glib::ustring & main_message, 
							Glib::ustring & secondary_message);
		
		
	protected:
		guint connection_a;
		guint connection_f;
		guint connection_bg_w;
		guint connection_bg_t;
		guint connection_bg_r_o;
		guint connection_bg_r_e;
		guint connection_x;
		guint connection_y;
		guint connection_w;
		guint connection_h;
		guint connection_autorun;
		guint old_window_width;
		guint old_window_height;
		guint old_window_xpos;
		guint old_window_ypos;
		void on_quit_button_clicked ();
		void on_help_about();
		void on_help_help();
		void on_all_off_clicked();
		void on_all_acpi_clicked();
		void on_all_save_clicked();
		void on_all_pause_clicked();
		void on_all_resume_clicked();
		void on_all_start_clicked();
		void on_all_console_clicked();
		void on_sensitivity_signal(std::vector<bool> sens);
		void on_font_selected();
		void on_window_x_changed(guint connectionid, const Gnome::Conf::Entry& entry);
		void on_window_y_changed(guint connectionid, const Gnome::Conf::Entry& entry);
		void on_window_w_changed(guint connectionid, const Gnome::Conf::Entry& entry);
		void on_window_h_changed(guint connectionid, const Gnome::Conf::Entry& entry);
		void on_font_external_changed(guint connectionid, const Gnome::Conf::Entry& entry);
		void on_colour_w_external_changed(guint connectionid, const Gnome::Conf::Entry& entry);
		void on_colour_t_external_changed(guint connectionid, const Gnome::Conf::Entry& entry);
		void on_colour_r_o_external_changed(guint connectionid, const Gnome::Conf::Entry& entry);
		void on_colour_r_e_external_changed(guint connectionid, const Gnome::Conf::Entry& entry);
		void on_autorun_external_toggled(guint connectionid, const Gnome::Conf::Entry& entry);
		void on_show_error_messages_external_toggled(guint connectionid, const Gnome::Conf::Entry& entry);
		void on_colour_w_changed();
		void on_colour_t_changed();
		void on_colour_r_o_changed();
		void on_colour_r_e_changed();
		void on_autorun_toggled();
		void on_show_error_messages_toggled();
		void make_autorun_file();
		bool on_timeout();
		
		Glib::ustring make_markup(Glib::ustring text);
		int timer_count;
		std::map<std::string,int> states;
	private:
	
};

#endif // _VB_MANAGE_HPP_
