/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * trunk
 * Copyright (C) Eoin McMahon 2010 <eoinmcmahon0@AT@gmail.com>
	 * 
 * trunk is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
	 * 
 * trunk is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _VIEW_ROW_HPP_
#define _VIEW_ROW_HPP_



#include <glib.h>
#include <iostream>
#include <vector>
#include <set>
#include <map>
#include <list>
#include <time.h>
#include <gtkmm.h>
#include <libglademm/xml.h>
#include <sys/wait.h>
#include <signal.h>
#include <fstream>
#include <gconfmm.h>

extern Glib::RefPtr<Gnome::Conf::Client> gconfclient;
extern Glib::RefPtr<Gdk::Screen> gscreen;
extern Glib::ustring fontname;
extern Glib::ustring bg_window;
extern Glib::ustring bg_title;
extern Glib::ustring bg_row_odd;
extern Glib::ustring bg_row_even;
extern bool show_error_messages;
extern const Glib::ustring opening;
extern const Glib::ustring middle;
extern const Glib::ustring closing;
extern const int hpadding;
#include "data.hpp"
#include "model-row.hpp"

using namespace std;
class ViewRow : public Data, public Gtk::HBox
{
	public:
	
		
		
		friend class View;
		friend class vbManage;
		size_t index;
		vector<STR> v_s_cols;
		vector<BOO> v_b_cols;
		ModelRow * pSource;
		//ViewRow();
		ViewRow(ModelRow  & source);
		
		//static void sigchld_hdl(int);
		struct sigaction child_action;
		struct sigaction * pOld_action;
		//~ STR get_str_value(v_s_col col);
		//~ BOO get_boo_value(v_b_col col);
		//~ void set_str_value(v_s_col col, STR value);
		//~ void set_boo_value(v_b_col col, BOO value);
		//~ vector<STR>::iterator get_str_iterator();
		//~ vector<BOO>::iterator get_boo_iterator();
		STR get_uuid();
		typedef sigc::signal<void, clock_t, Data::STR, m_b_col, Data::BOO, Data::BOO>
			type_BOO_changed_signal;
		type_BOO_changed_signal BOO_changed_ack_signal();
		typedef sigc::signal<void, clock_t, Data::STR, m_s_col, Data::STR, Data::STR>
			type_STR_changed_signal;
		type_STR_changed_signal STR_changed_ack_signal();
		typedef sigc::signal< void , clock_t, Data::STR, std::vector<int> & , int >
			type_width_changed_signal;
		type_width_changed_signal width_changed_signal();
		void get_width_changed_signal();
		
		vector<bool> get_button_sensitivities();
		
		virtual void emit_row_changed_STR_ack(clock_t timestamp,
									STR uuid,
									m_s_col col,
									STR old,
									STR newvalue);
		virtual void emit_row_changed_BOO_ack(clock_t timestamp,
									STR uuid,
									m_b_col col,
									BOO old,
									BOO newvalue);
		virtual void emit_width_changed_signal(clock_t timestamp,
											   Data::STR uuid, 
											   vector<int> & Width,
											   int height);
		static void sigchld_hdl (int sig)
			
	{
		
		/* Wait for all dead processes.
		 
		 * We use a non-blocking call to be sure this signal handler will not
		 
		 * block if a child was cleaned up in another part of the program. */
		
		while (waitpid(-1, NULL, WNOHANG) > 0) {
			
		}
		
	};
			
	
		
		
	protected:
		int maxheight;
		vector<int> widths;
		ModelRow * pModelRow;
		vector<Gtk::Widget*> widgets;
		STR uuid;
		STR mname;
		STR mstatus;
		STR mport;
		STR mautostart;
		std::vector<Gtk::Widget*> wlist;
		Gtk::Label vm;
		Gtk::Label status;
		Gtk::Label vrdp;
		Gtk::Button hardoff;
		Gtk::Button softoff;
		Gtk::Button save;
		Gtk::ToggleButton pause;
		Gtk::Button start;
		Gtk::Button console;
		Gtk::CheckButton autostart;
		Gtk::Label buffer;
		type_STR_changed_signal m_STR_changed_ack_signal;
		type_BOO_changed_signal m_BOO_changed_ack_signal;
		type_width_changed_signal m_width_changed_signal;
		void on_row_changed_STR(clock_t timestamp,
									STR uuid,
									m_s_col col,
									STR old,
									STR newvalue);
		void on_row_changed_BOO(clock_t timestamp,
									STR uuid,
									m_b_col col,
									BOO old,
									BOO newvalue);
		
		void on_width_ordered_signal(clock_t timestamp,
									 Data::STR uuid,
									 vector<int> & Width, int height);
		void on_console_button_clicked();
		void on_run_button_clicked();
		void on_pause_button_clicked();
		void on_save_button_clicked();
		void on_softoff_button_clicked();
		void on_hardoff_button_clicked();
		void on_autostart_toggled();
		void output_dialog(
						   Glib::ustring & main_message , 
						   Glib::ustring & secondary_message );
		Glib::ustring make_markup(Glib::ustring text);
		void on_font_change();
		void update_widths(bool emit_signal = true);
};



#endif // _VIEW_ROW_HPP_
