/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * trunk
 * Copyright (C) Eoin McMahon 2010 <eoinmcmahon0@AT@gmail.com>
	 * 
 * trunk is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
	 * 
 * trunk is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "view.hpp"


View::ViewTitle::ViewTitle(View & parent) : 
	Gtk::HBox(false, hpadding),
	vm(make_markup(tvm), Gtk::ALIGN_LEFT),
	status(make_markup(tstatus), Gtk::ALIGN_LEFT),
	vrdp(make_markup(tvrdp), Gtk::ALIGN_LEFT),
	hardoff(make_markup(thardoff), Gtk::ALIGN_LEFT),
	softoff(make_markup(tsoftoff), Gtk::ALIGN_LEFT),
	save(make_markup(tsave), Gtk::ALIGN_LEFT),
	pause(make_markup(tpause), Gtk::ALIGN_LEFT),
	resume(make_markup(tresume), Gtk::ALIGN_LEFT),
	start(make_markup(tstart), Gtk::ALIGN_LEFT),
	console(make_markup(tconsole), Gtk::ALIGN_LEFT),
	autostart(make_markup(tautostart), Gtk::ALIGN_LEFT),
	buffer(make_markup(tbuffer), Gtk::ALIGN_LEFT),
	myparent(&parent),
	wlist(),
	widths(),
	pWidth(&widths),
	maxheight(0),
	uuid(Glib::ustring("Title"))
{
	vm.set_use_markup();
	status.set_use_markup();
	vrdp.set_use_markup();
	start.set_use_markup();
	console.set_use_markup();
	softoff.set_use_markup();
	hardoff.set_use_markup();
	pause.set_use_markup();
	save.set_use_markup();
	autostart.set_use_markup();
	buffer.set_use_markup();
	
	Gtk::VSeparator * pVSeparatora = Gtk::manage( new Gtk::VSeparator);
	Gtk::VSeparator * pVSeparatorb = Gtk::manage( new Gtk::VSeparator);
	Gtk::VSeparator * pVSeparatorc = Gtk::manage( new Gtk::VSeparator);
	Gtk::VSeparator * pVSeparatord = Gtk::manage( new Gtk::VSeparator);
	Gtk::VSeparator * pVSeparatore = Gtk::manage( new Gtk::VSeparator);
	Gtk::VSeparator * pVSeparatorf = Gtk::manage( new Gtk::VSeparator);
	Gtk::VSeparator * pVSeparatorg = Gtk::manage( new Gtk::VSeparator);
	Gtk::VSeparator * pVSeparatorh = Gtk::manage( new Gtk::VSeparator);
	Gtk::VSeparator * pVSeparatori = Gtk::manage( new Gtk::VSeparator);
	
	pack_start(vm, false, false);
	pack_start(*pVSeparatora, false, false);
	pack_start(status, false, false);
	pack_start(*pVSeparatorb, false, false);
	pack_start(vrdp, false, false);
	pack_start(*pVSeparatorc, false, false);
	pack_start(start, false, false);
	pack_start(*pVSeparatord, false, false);
	pack_start(console, false, false);
	pack_start(*pVSeparatore, false, false);
	pack_start(softoff, false, false);
	pack_start(*pVSeparatorf, false, false);
	pack_start(hardoff, false, false);
	pack_start(*pVSeparatorg, false, false);
	pack_start(pause, false, false);
	pack_start(*pVSeparatorh, false, false);
	pack_start(save, false, false);
	pack_start(*pVSeparatori, false, false);
	pack_start(autostart, false, false);
	pack_start(buffer, false, false);
	
	wlist.push_back(&vm);
	wlist.push_back(&status);
	wlist.push_back(&vrdp);
	wlist.push_back(&start);
	wlist.push_back(&console);
	wlist.push_back(&softoff);
	wlist.push_back(&hardoff);
	wlist.push_back(&pause);
	wlist.push_back(&save);
	wlist.push_back(&autostart);
	wlist.push_back(&buffer);
	
	show_all_children();
	vector<Gtk::Widget*>::iterator it;
//	cout << "New View with Title row created"<<endl;
//	cout << "Widths: ";
	for ( it = wlist.begin() ;
		 it != wlist.end() ;
		 it++/*, cout << ", "*/)
	{
		widths.push_back((*it)->size_request().width);
//		cout << (*it)->size_request().width;
		maxheight = MAX(maxheight, (*it)->size_request().height);
	}
//	cout << "height ="<<maxheight<<endl;
	// connect width_changed_signal to parent
	//this->width_changed_signal().connect(sigc::mem_fun(parent, &View::View::on_width_changed));
	// report sizes to parent here
	// parent not fully instantiated yet
	//emit_width_changed_signal(this->uuid, pWidth, maxheight);
	// connect parent's widths changed signal to function
	//~ parent.width_ordered_signal().connect(
										  //~ sigc::mem_fun(*this,
														//~ &View::ViewTitle::on_width_ordered_signal));
	//Won't compile until parent signal is in place
	
}

Connection::sig_type_width 
View::ViewTitle::width_changed_signal()
{
	return m_width_changed_signal;
}


void View::ViewTitle::emit_width_changed_signal(clock_t timestamp,
												Data::STR uuid,
												vector<int> &Width,
												int height)
{
	m_width_changed_signal.emit(timestamp, uuid, Width, height);
}

void View::ViewTitle::on_width_ordered_signal(clock_t timestamp,
											  Data::STR uuid,
											  vector<int> & Width, int height)
{
//	cout << "Title on width ordered invoked: "<< timestamp << endl;
	vector<Gtk::Widget*>::iterator it;
	int id = 0;
	this->maxheight = height;
	for ( it = wlist.begin() ;
		 it != wlist.end() ;
		 it++, id++)
	{
		this->widths[id] = (Width)[id];
		(*it)->set_size_request((Width)[id], maxheight);
	}
}

void View::ViewTitle::get_width_changed_signal()
{
//	cout << "get width changed signal from View Title"<<endl;
	emit_width_changed_signal(clock(), this->uuid, *pWidth, maxheight);
}


Glib::ustring View::ViewTitle::make_markup(Glib::ustring text)
{
	return Glib::ustring(opening + fontname + middle + text + closing);
}

void View::ViewTitle::on_font_change()
{
	//expect global value fontname to have been changed before this fn is called
	vm.set_label(make_markup(tvm));
	status.set_label(make_markup(tstatus));
	vrdp.set_label(make_markup(tvrdp));
	hardoff.set_label(make_markup(thardoff));
	softoff.set_label(make_markup(tsoftoff));
	save.set_label(make_markup(tsave));
	pause.set_label(make_markup(tpause));
	start.set_label(make_markup(tstart));
	console.set_label(make_markup(tconsole));
	autostart.set_label(make_markup(tautostart));
	
	update_widths();
	
}

void View::ViewTitle::update_widths(bool emit_signal )
{
	
	vector<Gtk::Widget*>::iterator it;
	int id = 0;
	maxheight = 0;
	int widthSoFar = 0;
	for ( it = wlist.begin() ;
			 it != wlist.end() ;
			 it++, id++)
		{
			(*it)->set_size_request();
			Gtk::Requisition requires = (*it)->size_request();
			Gtk::Allocation allocate(widthSoFar, 0, requires.width, requires.height);
			widthSoFar += requires.width;
			(*it)->size_allocate(allocate);
			this->widths[id] = requires.width;
			maxheight = MAX(maxheight, requires.height);
		}
		vector<int> * pWidth = & widths;
	if (emit_signal)
	{
		emit_width_changed_signal(clock(), this->uuid, widths, maxheight);
	}
}

//*****************************
// end of ViewTitle definitions
//*****************************

View::View(MModel & model) : 
	Gtk::VBox(false,0),
	buffer(false,2),
	rows(),
	rowEventBoxes(),
	uuidToEb(),
	ebToVr(),
	maxheight(0),
	widths(Data::n_v_cols, 0),
	pWidth(&widths),
	row_ch_BOO_conn_map(),
	row_ch_BOO_ack_conn_map(),
	row_ch_BOO_ack_to_me_map(),
	row_ch_STR_conn_map(),
	row_ch_STR_ack_conn_map(),
	wi_ch_conn_map(),
	wi_or_conn_map(),
	pVtEventBox(),
	pThisEventBox(dynamic_cast<Gtk::EventBox *>(get_parent())),
	vt(*this),
	pModel(model),
	OR_sensitivities(7, false)
{
	//~ cout << bg_window << " window colour" << endl;
	//~ cout << bg_title << " title colour" << endl;
	//~ cout << bg_row_even << " row_even colour" << endl;
	//~ cout << bg_row_odd << " row_odd colour" << endl;
	pVtEventBox = Gtk::manage( new Gtk::EventBox);
	pVtEventBox->modify_bg(Gtk::STATE_ACTIVE, Gdk::Color(bg_title));
	pVtEventBox->modify_bg(Gtk::STATE_INSENSITIVE, Gdk::Color(bg_title));
	pVtEventBox->modify_bg(Gtk::STATE_PRELIGHT, Gdk::Color(bg_title));
	pVtEventBox->modify_bg(Gtk::STATE_NORMAL, Gdk::Color(bg_title));
	pVtEventBox->modify_bg(Gtk::STATE_SELECTED, Gdk::Color(bg_title));
	
	pVtEventBox->add(vt);
	pack_start(*pVtEventBox, false, false);
	Gtk::HSeparator * pHSeparator = Gtk::manage( new Gtk::HSeparator);
	pHSeparator->set_size_request(-1,1);
	Gtk::EventBox * pHsEventBox = Gtk::manage( new Gtk::EventBox);
	pHsEventBox->modify_bg(Gtk::STATE_NORMAL, Gdk::Color(bg_title));
	pHsEventBox->add(*pHSeparator);
	pHsEventBox->set_size_request(-1,1);
	pack_start(*pHsEventBox, false, false, 0);
	pack_end(buffer);
	widths.push_back(0);
	// connect to signals incoming from m-model 
	model_row_added_connection = model.row_added_signal()
		.connect( sigc::mem_fun( *this, &View::on_row_added));
	model_row_deleted_connection = model.row_deleted_signal()
		.connect( sigc::mem_fun( *this, &View::on_row_deleted));
	
	//connect row added/deleted ack connections to model
	my_row_added_ack_connection = row_added_ack_signal ()
		.connect( sigc::mem_fun(model, &MModel::on_row_added_ack ));
	my_row_deleted_ack_connection = row_deleted_ack_signal ()
		.connect( sigc::mem_fun(model, &MModel::on_row_deleted_ack ));
	
	//connect vt's width changed signal to me
	incoming_width_changeds.insert(Connection::pair_type_width((Gtk::HBox *)&vt,
		  vt.width_changed_signal()
			.connect( sigc::mem_fun( *this, &View::on_width_changed))));
	
	// connect view title to my width ordered signal
	outgoing_width_ordereds.insert(Connection::pair_type_width((Gtk::HBox *)&vt,
	width_ordered_signal()
			.connect(sigc::mem_fun(vt, &View::ViewTitle::on_width_ordered_signal))));
	vt.get_width_changed_signal();
}

void View::on_width_changed(clock_t timestamp,
							Data::STR uuid,
							vector<int> &Width,
							int height)
{
	bool increased = true; //changed to always emit ordered signal
	if (true) // changed algorithm to always update from existing rows, or sizes
				// can never decrease.
	{
		vt.update_widths(false);
		widths.assign(vt.widths.begin(), vt.widths.end());
		maxheight = vt.maxheight;
		
		vector<ViewRow * >::iterator rowIt;
		vector<int>::iterator it;
		vector<int>::iterator copyIt;
		for (rowIt = rows.begin() ; rowIt != rows.end() ; rowIt++)
		{
			(*rowIt)->update_widths(false);
			maxheight = MAX(maxheight, (*rowIt)->maxheight);
			for (it = widths.begin(), copyIt = (*rowIt)->widths.begin() ;
				 ((it != widths.end()) && (copyIt != (*rowIt)->widths.end())) ;
				 it++, copyIt++)
			{
				(*it) = MAX( (*it), (*copyIt));
			}
		}
	}
	else
	{
		vector<int>::iterator it;
		vector<int>::iterator copyit;
		bool increased = true; //changed to always emit ordered signal
		if (height > maxheight ) { increased = true; }
		
		maxheight = MAX( height, maxheight);
		for ( it = widths.begin(), copyit = Width.begin() ;
			 ((it != widths.end()) && (copyit != Width.end())) ; 
			 copyit++ , it++ )
		{
			if ( (*copyit) > (*it) ) 
			{ 
				increased = true; 
			}
			(*it) = MAX( (*it) , (*copyit) );
		}
	}
	vector<ViewRow *>::iterator rowit;
	std::string nname = "Title";
	for ( rowit = rows.begin() ; rowit != rows.end() ; rowit++)
	{
		if ((*rowit)->uuid == uuid)
		{
			nname = (*rowit)->mname;
			break;
		}
	}
	
	//~ cout << "View has received width request from view row named "<<nname<<endl;
	//~ cout << "Timestamp: " << timestamp << endl;
	//~ cout << "widths.size= "<< widths.size() << " pWidth->size() = ";
	//~ cout << pWidth->size()<< endl;
	//~ cout << "Widths: ";
	//~ for ( it = widths.begin() ;
		 //~ it != widths.end() ;
		 //~ it++/**/, cout << ", "/**/)
	//~ {
		
		//~ cout << (*it);
		
	//~ }
	//~ cout << "height ="<<maxheight<<" Increased? " << 
		//~ (increased?"true":"false") << endl;
	
	if (increased)
	{
		emit_width_ordered_signal(clock(), "VIEW", (this->widths), maxheight);
	}
}

void View::on_row_added(clock_t timestamp, Data::STR uuid, ModelRow & mrow)
{
	//create new ViewRow and add it to this
	Gtk::EventBox * pViewRowEventBox = Gtk::manage( new Gtk::EventBox);
	Gdk::Color bg;
	if ( rows.size() % 2 == 0 )
	{
		bg = Gdk::Color(bg_row_odd);//because user counts from 1, computer from 0
	}
	else
	{
		bg = Gdk::Color(bg_row_even);//because user counts from 1, computer from 0
	}
	
	rowEventBoxes.insert(rowEventBoxes.end(), pViewRowEventBox);
	uuidToEb.insert(uuidEb(uuid, pViewRowEventBox));
	pViewRowEventBox->modify_bg(Gtk::STATE_NORMAL, bg);
	ViewRow * pvrow = Gtk::manage( new ViewRow(mrow));
	pViewRowEventBox->add(*pvrow);
	rows.insert(rows.end(), pvrow);
	ebToVr.insert(ebVr(pViewRowEventBox, pvrow));
	button_sensitivities.push_back(pvrow->get_button_sensitivities());
	calculate_sensitivities();
	pack_start(*pViewRowEventBox, false, false);
	Gtk::EventBox * pHsEventBox = Gtk::manage( new Gtk::EventBox);
	Gtk::HSeparator * pHSeparator = Gtk::manage( new Gtk::HSeparator);
	pHsEventBox->modify_bg(Gtk::STATE_NORMAL, bg);
	pHsEventBox->add(*pHSeparator);	
	pack_start(*pHsEventBox, false, false);
	//connect View Row signals to ModelRow and store connections
	sigc::connection conn;
	conn = pvrow->STR_changed_ack_signal()
		.connect(sigc::mem_fun(mrow,&ModelRow::on_row_changed_STR_ack));
	Connection::pair_type_value_STR row_ch_STR_ack_pair = make_pair(&mrow, conn);
	row_ch_STR_ack_conn_map.insert(row_ch_STR_ack_pair);
	
	conn = pvrow->BOO_changed_ack_signal()
		.connect(sigc::mem_fun(mrow,&ModelRow::on_row_changed_BOO_ack));
	Connection::pair_type_value_BOO row_ch_BOO_ack_pair = make_pair(&mrow, conn);
	row_ch_BOO_ack_conn_map.insert(row_ch_BOO_ack_pair);
	
	
	//connect BOO-ack signal to me so I can update OR_sensitivities
	conn = pvrow->BOO_changed_ack_signal()
		.connect(sigc::mem_fun(*this,&View::on_row_changed_BOO_ack));
	Connection::pair_type_value_BOO row_ch_BOO_ack_to_me_pair =
		make_pair(&mrow, conn);
	row_ch_BOO_ack_to_me_map.insert(row_ch_BOO_ack_to_me_pair);
	
	//connect Model Row signals to View Row and store connections
	conn = mrow.STR_changed_signal()
		.connect(sigc::mem_fun(pvrow,&ViewRow::on_row_changed_STR));
	Connection::pair_type_value_STR row_ch_STR_pair = make_pair(&mrow, conn);
	row_ch_STR_conn_map.insert(row_ch_STR_pair);
	
	conn = mrow.BOO_changed_signal()
		.connect(sigc::mem_fun(pvrow,&ViewRow::on_row_changed_BOO));
	Connection::pair_type_value_BOO row_ch_BOO_pair = make_pair(&mrow, conn);
	row_ch_BOO_conn_map.insert(row_ch_BOO_pair);
	
	// connect View Row width change request signals to this 
	conn = pvrow->width_changed_signal()
		.connect(sigc::mem_fun(*this, &View::on_width_changed));
	Connection::pair_type_width incoming_width_changeds_pair = 
		make_pair((Gtk::HBox *)pvrow, conn);
	incoming_width_changeds.insert(incoming_width_changeds_pair);
	
	// connect my row-width order signals to new view row
	conn = this->width_ordered_signal()
		.connect(sigc::mem_fun(pvrow, &ViewRow::on_width_ordered_signal));
	Connection::pair_type_width outgoing_width_ordereds_pair = 
		make_pair((Gtk::HBox *)pvrow, conn);
	outgoing_width_ordereds.insert(outgoing_width_ordereds_pair);
	
	pvrow->get_width_changed_signal();
	show_all_children();
	//cout << "Added view row" << endl;
	// send ack signal
	emit_row_added_ack_signal(timestamp, uuid, mrow);
}

void View::on_row_changed_BOO_ack(clock_t timestamp,
									Data::STR uuid,
									Data::m_b_col col,
									Data::BOO old,
									Data::BOO newvalue)
{
	//find the row by uuid, get a pointer to the BOO vector
	vector<ViewRow::ViewRow * >::iterator rowIt;
	ViewRow::ViewRow * pViewRow;
	bool foundViewRow = false;
	std::vector< std::vector <bool> >::iterator sensIt;
	for (rowIt = rows.begin(), sensIt = button_sensitivities.begin() ;
		 rowIt != rows.end() ; rowIt++, sensIt++ )
	{
		if ((*rowIt)->uuid ==  uuid)
		{
			(*sensIt) = (*rowIt)->get_button_sensitivities();
			break;
		}
		
	}
	calculate_sensitivities ();
}

void View::on_row_deleted(clock_t timestamp, Data::STR uuid, ModelRow & mrow)
{
	
	// find the event box from map
	std::vector<Gtk::EventBox *>::iterator ebIt;
	Gtk::EventBox * pEb = uuidToEb.find(uuid)->second;
	for (ebIt = rowEventBoxes.begin() ; ebIt != rowEventBoxes.end() ; ebIt++)
	{
		if (pEb == (*ebIt)) break;
	}
	if (ebIt == rowEventBoxes.end())
	{
		cout << "Error! In View::on_row_deleted, pEb not in rowEventBoxes"<< endl;
	}
	// find iterator to the view row from map
	std::vector<ViewRow::ViewRow *>::iterator vrIt;
	
	ViewRow::ViewRow * pVr = ebToVr.find(pEb)->second;
	for (vrIt = rows.begin() ; vrIt != rows.end() ; vrIt++)
	{
		if (pVr == (*vrIt)) break;
	}
	if (vrIt == rows.end())
	{
		cout << "Error! In View::on_row_deleted, pVr not in rows"<< endl;
	}
	
	// delete the connections 
	// find seven connections on lists by ModelRow and disconnect 
	Connection::it_type_value_BOO BOOit;
	Connection::it_type_value_STR STRit;
	Connection::it_type_width WIDit;
	BOOit = row_ch_BOO_conn_map.find (&mrow);
	if (BOOit != row_ch_BOO_conn_map.end() )
	{
		(BOOit->second).disconnect();
	}
	BOOit = row_ch_BOO_ack_conn_map.find (&mrow);
	if (BOOit != row_ch_BOO_ack_conn_map.end() )
	{
		(BOOit->second).disconnect();
	}
	BOOit = row_ch_BOO_ack_to_me_map.find (&mrow);
	if (BOOit != row_ch_BOO_ack_to_me_map.end() )
	{
		(BOOit->second).disconnect();
	}
	STRit = row_ch_STR_conn_map.find (&mrow);
	if (STRit != row_ch_STR_conn_map.end() )
	{
		(STRit->second).disconnect();
	}
	STRit = row_ch_STR_ack_conn_map.find (&mrow);
	if (STRit != row_ch_STR_ack_conn_map.end() )
	{
		(STRit->second).disconnect();
	}
	WIDit = wi_ch_conn_map.find ((Gtk::HBox *)pVr);
	if (WIDit != wi_ch_conn_map.end() )
	{
		(WIDit->second).disconnect();
	}
	WIDit = wi_or_conn_map.find ((Gtk::HBox *)pVr);
	if (WIDit != wi_or_conn_map.end() )
	{
		(WIDit->second).disconnect();
	}
	// delete the view row from rows and Event Box
	rows.erase(vrIt);
	pEb->remove();
	delete(pVr);
	// delete the event box from uuidToEb, ebToVr, rowEventBoxes and View 
	uuidToEb.erase(uuid);
	ebToVr.erase(pEb);
	rowEventBoxes.erase(ebIt);
	remove(*pEb);
	delete(pEb);
	on_bg_row_even_change ();
	on_bg_row_odd_change ();
	
	// send ack signal
	emit_row_deleted_ack_signal(timestamp, uuid, mrow);
	
}

void View::calculate_sensitivities()
{
	std::vector<bool> temp(7, false);
	std::vector<std::vector<bool> >::iterator rIt;
	for ( rIt = button_sensitivities.begin() ; 
		  rIt != button_sensitivities.end() ; rIt++ )
	{
		for (int index = 0 ; index != 7 ; index++ )
		{
			temp[index] = ( (*rIt)[index] || temp[index] );
		}
	}
	OR_sensitivities = temp;
	emit_sensitivity_signal(OR_sensitivities);
}

void View::emit_row_added_ack_signal(clock_t timestamp, Data::STR uuid, ModelRow &mrow)
{
	m_row_added_ack_signal.emit(timestamp, uuid, mrow);
}

void View::emit_row_deleted_ack_signal(clock_t timestamp, Data::STR uuid, ModelRow & mrow)
{
	m_row_deleted_ack_signal.emit(timestamp, uuid, mrow);
}

void View::emit_sensitivity_signal(std::vector<bool> sensitivity)
{
	m_sensitivity_signal.emit(sensitivity);
}

void View::emit_width_ordered_signal(clock_t timestamp,
									 Data::STR uuid,
									 vector<int> & Width, int height)
{
	m_width_ordered_signal.emit(clock(), "View",   Width, height);
}

Connection::sig_type_width View::width_ordered_signal()
{
	return m_width_ordered_signal;
}

Connection::sig_type_row View::row_added_ack_signal()
{
	return m_row_added_ack_signal;
}

Connection::sig_type_row View::row_deleted_ack_signal()
{
	return m_row_deleted_ack_signal;
}
Connection::sig_type_sensitivity View::sensitivity_signal()
{
	return m_sensitivity_signal;
}
void View::on_font_change()
{
	std::vector<ViewRow::ViewRow *>::iterator rowIt;
	for ( rowIt = rows.begin() ; rowIt != rows.end() ; rowIt++)
	{
		(*rowIt)->on_font_change();
	}
	vt.on_font_change();
}

void View::on_bg_window_change()
{
	pThisEventBox->modify_bg(Gtk::STATE_NORMAL, Gdk::Color(bg_window));
}

void View::on_bg_title_change()
{
	pVtEventBox->modify_bg(Gtk::STATE_NORMAL, Gdk::Color(bg_title));
}

void View::on_bg_row_odd_change()
{
	std::vector<Gtk::EventBox *>::iterator ebIt;
	size_t count;
	// select rows with even indices starting from 0 = odd ones counting from 1
	for ( count = 0 ; count < rowEventBoxes.size() ; count +=2)
	{
		ebIt = rowEventBoxes.begin() + count;
		(*ebIt)->modify_bg(Gtk::STATE_NORMAL, Gdk::Color(bg_row_odd));
	}
}

void View::on_bg_row_even_change()
{
	std::vector<Gtk::EventBox *>::iterator ebIt;
	size_t count;
	// select rows with odd indices starting from 0 = even ones counting from 1
	for ( count = 1 ; count < rowEventBoxes.size() ; count +=2)
	{
		ebIt = rowEventBoxes.begin() + count;
		(*ebIt)->modify_bg(Gtk::STATE_NORMAL, Gdk::Color(bg_row_even));
	}
}


const Glib::ustring View::ViewTitle::tvm("Machine");
const Glib::ustring View::ViewTitle::tstatus("Status");
const Glib::ustring View::ViewTitle::tvrdp("Port");
const Glib::ustring View::ViewTitle::thardoff("Power Button");
const Glib::ustring View::ViewTitle::tsoftoff("Send Power Signal");
const Glib::ustring View::ViewTitle::tsave("Save");
const Glib::ustring View::ViewTitle::tpause("Pause");
const Glib::ustring View::ViewTitle::tresume("Resume");
const Glib::ustring View::ViewTitle::tstart("Start");
const Glib::ustring View::ViewTitle::tconsole("Console");
const Glib::ustring View::ViewTitle::tautostart("Autostart");
const Glib::ustring View::ViewTitle::tbuffer("");
