/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * trunk
 * Copyright (C) Eoin McMahon 2010 <eoinmcmahon0@AT@gmail.com>
 * 
 * trunk is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * trunk is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _VIEW_HPP_
#define _VIEW_HPP_

#include <glib.h>
#include <iostream>
#include <vector>
#include <map>
#include <list>
#include <time.h>
#include <gtkmm.h>
#include <libglademm/xml.h>
#include <gconfmm.h>

extern Glib::RefPtr<Gnome::Conf::Client> gconfclient;
extern Glib::RefPtr<Gdk::Screen> gscreen;
extern Glib::ustring fontname;
extern Glib::ustring bg_window;
extern Glib::ustring bg_title;
extern Glib::ustring bg_row_odd;
extern Glib::ustring bg_row_even;
extern const Glib::ustring opening;
extern const Glib::ustring middle;
extern const Glib::ustring closing;
extern const int hpadding;

#include "view-row.hpp"
#include "m-model.hpp"
#include "model-row.hpp"



class View: public Gtk::VBox 
{
	public:
		
		class ViewTitle : public Gtk::HBox
	{
		public:
			ViewTitle(View & parent);
			std::vector<Gtk::Widget*> wlist;
			std::vector<int> widths;
			std::vector<int> * pWidth;
			int maxheight;
			View * myparent;
			static const Glib::ustring tvm;
			static const Glib::ustring tstatus;
			static const Glib::ustring tvrdp;
			static const Glib::ustring thardoff;
			static const Glib::ustring tsoftoff;
			static const Glib::ustring tsave;
			static const Glib::ustring tpause;
			static const Glib::ustring tresume;
			static const Glib::ustring tstart;
			static const Glib::ustring tconsole;
			static const Glib::ustring tautostart;
			static const Glib::ustring tbuffer;
			
			
			Data::STR uuid;
			Gtk::Label vm;
			Gtk::Label status;
			Gtk::Label vrdp;
			Gtk::Label hardoff;
			Gtk::Label softoff;
			Gtk::Label save;
			Gtk::Label pause;
			Gtk::Label resume;
			Gtk::Label start;
			Gtk::Label console;
			Gtk::Label autostart;
			Gtk::Label buffer;
		
			Connection::sig_type_width width_changed_signal();
			void get_width_changed_signal();
			virtual void emit_width_changed_signal(clock_t timestamp,
												   Data::STR uuid, 
											   vector<int> &Width,
												   int height);
			void on_width_ordered_signal(clock_t timestamp,
										 Data::STR uuid,
										 std::vector<int> & Width, int height);
			Glib::ustring make_markup(Glib::ustring text);
			void on_font_change();
			void update_widths(bool emit_signal = true );
		protected:
			Connection::sig_type_width m_width_changed_signal;
	};
		
		
		std::vector<ViewRow::ViewRow *> rows;
		std::vector<Gtk::EventBox *> rowEventBoxes;
		typedef pair<Gtk::EventBox * , ViewRow::ViewRow * > ebVr;
		std::map<Gtk::EventBox * , ViewRow::ViewRow * > ebToVr;
		typedef pair<Data::STR  , Gtk::EventBox * > uuidEb;
		std::map<Data::STR  , Gtk::EventBox * > uuidToEb;
		std::vector<std::vector<bool> > button_sensitivities;
		void calculate_sensitivities();
		std::vector<bool> OR_sensitivities;
		Gtk::EventBox * pVtEventBox;
		Gtk::EventBox * pThisEventBox;
		ViewTitle vt;
		Gtk::HBox buffer;
		MModel & pModel;
		View(MModel & model);
		
		Connection::sig_type_width width_ordered_signal();
		virtual void emit_width_ordered_signal(clock_t timestamp,
											   Data::STR uuid,
											   vector<int> &Width, int height);
		
		Connection::sig_type_row row_added_ack_signal();
		Connection::sig_type_row row_deleted_ack_signal();
		
		//typedef sigc::signal< void, std::vector<bool> > sensitivity_signal_type;
		Connection::sig_type_sensitivity sensitivity_signal();
		
		//typedef ViewTitle::type_width_changed_signal::iterator wi_ch_it;
		//typedef ViewTitle::type_width_changed_signal& wi_ch_ref;
		//typedef vector< pair<ViewRow * , sigc::connection > > wi_ch_vector_type;
		//typedef type_width_ordered_signal::iterator wi_or_it;
		//typedef type_width_ordered_signal& wi_or_ref;
		//typedef vector< pair<ViewRow * , sigc::connection > > wi_or_vector_type;
		//typedef ViewRow::type_BOO_changed_signal::iterator BOO_ch_it;
		//typedef ViewRow::type_STR_changed_signal::iterator STR_ch_it;
		
		//typedef vector< pair<ModelRow * , sigc::connection > > BOO_ch_vector_type;
		//typedef vector< pair<ModelRow * , sigc::connection > > STR_ch_vector_type;
		Connection::map_type_width wi_ch_conn_map;
		Connection::map_type_width wi_or_conn_map;
		Connection::map_type_value_BOO row_ch_BOO_conn_map;
		Connection::map_type_value_BOO row_ch_BOO_ack_conn_map;
		Connection::map_type_value_BOO row_ch_BOO_ack_to_me_map;
		Connection::map_type_value_STR row_ch_STR_conn_map;
		Connection::map_type_value_STR row_ch_STR_ack_conn_map;
		
		virtual void emit_row_added_ack_signal(clock_t timestamp,
											   Data::STR uuid, ModelRow & mrow);
		virtual void emit_row_deleted_ack_signal(clock_t timestamp,
												 Data::STR uuid, ModelRow & mrow);
		virtual void emit_sensitivity_signal(std::vector<bool> sensitivities);
		void on_font_change();
		void on_bg_window_change();
		void on_bg_title_change();
		void on_bg_row_odd_change();
		void on_bg_row_even_change();
		
protected:
		sigc::connection model_row_added_connection;
		sigc::connection model_row_deleted_connection;
		sigc::connection my_row_added_ack_connection;
		sigc::connection my_row_deleted_ack_connection;
		Connection::map_type_width incoming_width_changeds;
		Connection::map_type_width outgoing_width_ordereds;
		int maxheight;
		vector<int> widths;
		vector<int> * pWidth;
		Connection::sig_type_width m_width_ordered_signal;
		Connection::sig_type_row m_row_added_ack_signal;
		Connection::sig_type_row m_row_deleted_ack_signal;
		Connection::sig_type_sensitivity m_sensitivity_signal;
		void on_width_changed(
							  clock_t timestamp,
							  Data::STR uuid,
							  vector<int> &Width,
							  int height);
		void on_row_added(clock_t timestamp, Data::STR uuid, ModelRow & mrow);
		void on_row_deleted(clock_t timestamp, Data::STR uuid, ModelRow & mrow);
		void on_row_changed_BOO_ack(clock_t timestamp,
									Data::STR uuid,
									Data::m_b_col col,
									Data::BOO old,
									Data::BOO newvalue);
private:

};



#endif // _VIEW_HPP_
